#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import KeyboardButtonRequestManagedBot
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def keyboard_button_request_managed_bot():
    return KeyboardButtonRequestManagedBot(
        KeyboardButtonRequestManagedBotTestBase.request_id,
        KeyboardButtonRequestManagedBotTestBase.suggested_name,
        KeyboardButtonRequestManagedBotTestBase.suggested_username,
    )


class KeyboardButtonRequestManagedBotTestBase:
    request_id = 4
    suggested_name = "suggested_name"
    suggested_username = "username"


class TestKeyboardButtonRequestManagedBotWithoutRequest(KeyboardButtonRequestManagedBotTestBase):
    def test_slot_behaviour(self, keyboard_button_request_managed_bot):
        inst = keyboard_button_request_managed_bot
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "request_id": self.request_id,
            "suggested_name": self.suggested_name,
            "suggested_username": self.suggested_username,
        }
        keyboard_button_request_managed_bot = KeyboardButtonRequestManagedBot.de_json(
            json_dict, offline_bot
        )
        assert keyboard_button_request_managed_bot.api_kwargs == {}
        assert keyboard_button_request_managed_bot.request_id == self.request_id
        assert keyboard_button_request_managed_bot.suggested_name == self.suggested_name
        assert keyboard_button_request_managed_bot.suggested_username == self.suggested_username

    def test_to_dict(self):
        keyboard_button_request_managed_bot = KeyboardButtonRequestManagedBot(
            self.request_id,
            self.suggested_name,
            self.suggested_username,
        )
        keyboard_button_request_managed_bot_dict = keyboard_button_request_managed_bot.to_dict()
        assert (
            keyboard_button_request_managed_bot_dict["request_id"]
            == keyboard_button_request_managed_bot.request_id
        )
        assert (
            keyboard_button_request_managed_bot_dict["suggested_name"]
            == keyboard_button_request_managed_bot.suggested_name
        )
        assert (
            keyboard_button_request_managed_bot_dict["suggested_username"]
            == keyboard_button_request_managed_bot.suggested_username
        )

    def test_equality(self):
        a = KeyboardButtonRequestManagedBot(
            self.request_id, self.suggested_name, self.suggested_username
        )
        b = KeyboardButtonRequestManagedBot(
            self.request_id, self.suggested_name, self.suggested_username
        )
        c = KeyboardButtonRequestManagedBot(1, self.suggested_name, self.suggested_username)
        d = "not a KeyboardButtonRequestManagedBot"

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that tests a Telegram LivePhoto."""

import asyncio
import datetime as dtm
import os
from pathlib import Path

import pytest

from telegram import (
    InputFile,
    LivePhoto,
    MessageEntity,
    PhotoSize,
    ReplyParameters,
    Voice,
)
from telegram.constants import ParseMode
from telegram.error import BadRequest, TelegramError
from telegram.helpers import escape_markdown
from telegram.request import RequestData
from tests.auxil.build_messages import make_message
from tests.auxil.files import data_file
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def live_photo():
    return LivePhoto(
        file_id=LivePhotoTestBase.file_id,
        file_unique_id=LivePhotoTestBase.file_unique_id,
        width=LivePhotoTestBase.width,
        height=LivePhotoTestBase.height,
        duration=LivePhotoTestBase.duration,
        photo=LivePhotoTestBase.photo,
        mime_type=LivePhotoTestBase.mime_type,
        file_size=LivePhotoTestBase.file_size,
    )


class LivePhotoTestBase:
    caption = "LivePhotoTest - *Caption*"
    width = 360
    height = 640
    duration = dtm.timedelta(seconds=5)
    file_size = 326534
    mime_type = "video/mp4"
    photo = (PhotoSize("file_id", "unique_id", 640, 360, file_size=0),)
    file_id = "5a3128a4d2a04750b5b58397f3b5e812"
    file_unique_id = "adc3145fd2e84d95b64d68eaa22aa33e"


class TestLivePhotoWithoutRequest(LivePhotoTestBase):
    def test_slot_behaviour(self, live_photo):
        for attr in live_photo.__slots__:
            assert getattr(live_photo, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(live_photo)) == len(set(mro_slots(live_photo))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "file_id": self.file_id,
            "file_unique_id": self.file_unique_id,
            "width": self.width,
            "height": self.height,
            "duration": int(self.duration.total_seconds()),
            "mime_type": self.mime_type,
            "file_size": self.file_size,
            "photo": [photo_size.to_dict() for photo_size in self.photo],
        }
        json_live_photo = LivePhoto.de_json(json_dict, offline_bot)
        assert json_live_photo.api_kwargs == {}

        assert json_live_photo.file_id == self.file_id
        assert json_live_photo.file_unique_id == self.file_unique_id
        assert json_live_photo.width == self.width
        assert json_live_photo.height == self.height
        assert json_live_photo.duration == self.duration
        assert json_live_photo.mime_type == self.mime_type
        assert json_live_photo.file_size == self.file_size
        assert json_live_photo.photo == self.photo

    def test_to_dict(self, live_photo):
        live_photo_dict = live_photo.to_dict()

        assert isinstance(live_photo_dict, dict)
        assert live_photo_dict["file_id"] == live_photo.file_id
        assert live_photo_dict["file_unique_id"] == live_photo.file_unique_id
        assert live_photo_dict["width"] == live_photo.width
        assert live_photo_dict["height"] == live_photo.height
        assert live_photo_dict["duration"] == int(self.duration.total_seconds())
        assert isinstance(live_photo_dict["duration"], int)
        assert live_photo_dict["mime_type"] == live_photo.mime_type
        assert live_photo_dict["file_size"] == live_photo.file_size
        assert live_photo_dict["photo"] == [p.to_dict() for p in self.photo]

    def test_equality(self, live_photo):
        a = LivePhoto(
            live_photo.file_id, live_photo.file_unique_id, self.width, self.height, self.duration
        )
        b = LivePhoto("", live_photo.file_unique_id, self.width, self.height, self.duration)
        c = LivePhoto(live_photo.file_id, live_photo.file_unique_id, 0, 0, 0)
        d = LivePhoto("", "", self.width, self.height, self.duration)
        e = Voice(live_photo.file_id, live_photo.file_unique_id, self.duration)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

    async def test_send_with_live_photo(self, monkeypatch, offline_bot, chat_id, live_photo):
        async def make_assertion(url, request_data: RequestData, *args, **kwargs):
            data = request_data.parameters
            return (
                data["live_photo"] == live_photo.file_id
                and data["photo"] == live_photo.photo[0].file_id
            )

        monkeypatch.setattr(offline_bot.request, "post", make_assertion)

        assert await offline_bot.send_live_photo(
            chat_id=chat_id,
            live_photo=live_photo,
            photo=live_photo.photo[0],
        )

    @pytest.mark.parametrize(
        ("default_bot", "custom"),
        [
            ({"parse_mode": ParseMode.HTML}, None),
            ({"parse_mode": ParseMode.HTML}, ParseMode.MARKDOWN_V2),
            ({"parse_mode": None}, ParseMode.MARKDOWN_V2),
        ],
        indirect=["default_bot"],
    )
    async def test_send_live_photo_default_quote_parse_mode(
        self, default_bot, chat_id, live_photo, custom, monkeypatch
    ):
        async def make_assertion(url, request_data: RequestData, *args, **kwargs):
            assert request_data.parameters["reply_parameters"].get("quote_parse_mode") == (
                custom or default_bot.defaults.quote_parse_mode
            )
            return make_message("dummy reply").to_dict()

        kwargs = {"message_id": "1"}
        if custom is not None:
            kwargs["quote_parse_mode"] = custom

        monkeypatch.setattr(default_bot.request, "post", make_assertion)
        await default_bot.send_live_photo(
            chat_id=chat_id,
            live_photo=live_photo,
            photo=live_photo.photo[0],
            reply_parameters=ReplyParameters(**kwargs),
        )

    @pytest.mark.parametrize("local_mode", [True, False])
    async def test_send_live_photo(
        self, dummy_message_dict, monkeypatch, offline_bot, chat_id, local_mode
    ):
        try:
            offline_bot._local_mode = local_mode
            # For just test that the correct paths are passed as we have no local Bot API set up
            test_flag = False
            photo = data_file("telegram.jpg")
            expected_photo = photo.as_uri()

            live_photo = data_file("telegram.mp4")
            expected_live_photo = live_photo.as_uri()

            async def make_assertion(_, data, *args, **kwargs):
                nonlocal test_flag
                if local_mode:
                    test_flag = (
                        data.get("live_photo") == expected_live_photo
                        and data.get("photo") == expected_photo
                    )
                else:
                    test_flag = isinstance(data.get("live_photo"), InputFile) and isinstance(
                        data.get("photo"), InputFile
                    )
                return dummy_message_dict

            monkeypatch.setattr(offline_bot, "_post", make_assertion)
            await offline_bot.send_live_photo(chat_id, live_photo=live_photo, photo=photo)
            assert test_flag
        finally:
            offline_bot._local_mode = False


class TestLivePhotoWithRequest(LivePhotoTestBase):
    async def test_error_send_empty_file(self, bot, chat_id):
        with Path(os.devnull).open("rb") as f, pytest.raises(TelegramError):
            await bot.send_live_photo(chat_id=chat_id, live_photo=f, photo=f)

    async def test_error_send_empty_file_id(self, bot, chat_id):
        with pytest.raises(TelegramError):
            await bot.send_live_photo(chat_id=chat_id, live_photo="", photo="")

    async def test_get_and_download(self, bot, real_live_photo, tmp_file):
        new_file = await bot.get_file(real_live_photo.file_id)

        assert new_file.file_size == real_live_photo.file_size
        assert new_file.file_unique_id == real_live_photo.file_unique_id
        assert new_file.file_path.startswith("https://")

        await new_file.download_to_drive(tmp_file)

        assert tmp_file.is_file()

    async def test_send_resend(self, bot, chat_id, real_live_photo, photo_file):
        message = await bot.send_live_photo(
            chat_id=chat_id, live_photo=real_live_photo.file_id, photo=photo_file
        )
        assert message.live_photo == real_live_photo

    async def test_send_all_args(self, bot, chat_id, video_file, live_photo, photo_file):
        message = await bot.send_live_photo(
            chat_id,
            live_photo=video_file,
            photo=photo_file,
            caption=self.caption,
            disable_notification=False,
            protect_content=True,
            filename="telegram_custom.png",
            parse_mode="Markdown",
        )

        assert isinstance(message.live_photo, LivePhoto)
        assert isinstance(message.live_photo.file_id, str)
        assert message.live_photo.file_id
        assert isinstance(message.live_photo.file_unique_id, str)
        assert message.live_photo.file_unique_id
        assert message.live_photo.photo
        assert isinstance(message.live_photo.photo[0], PhotoSize)
        assert message.live_photo.mime_type == live_photo.mime_type
        assert message.live_photo.file_size == live_photo.file_size
        assert message.caption == self.caption.replace("*", "")
        assert message.has_protected_content

    @pytest.mark.parametrize("default_bot", [{"protect_content": True}], indirect=True)
    async def test_send_live_photo_default_protect_content(
        self,
        chat_id,
        default_bot,
        real_live_photo,
    ):
        tasks = asyncio.gather(
            default_bot.send_live_photo(
                chat_id, photo=real_live_photo.photo[0], live_photo=real_live_photo
            ),
            default_bot.send_live_photo(
                chat_id,
                photo=real_live_photo.photo[0],
                live_photo=real_live_photo,
                protect_content=False,
            ),
        )
        protected, unprotected = await tasks
        assert protected.has_protected_content
        assert not unprotected.has_protected_content

    async def test_send_live_photo_caption_entities(self, bot, chat_id, video_file, photo_file):
        test_string = "Italic Bold Code"
        entities = [
            MessageEntity(MessageEntity.ITALIC, 0, 6),
            MessageEntity(MessageEntity.ITALIC, 7, 4),
            MessageEntity(MessageEntity.ITALIC, 12, 4),
        ]
        message = await bot.send_live_photo(
            chat_id,
            photo=photo_file,
            live_photo=video_file,
            caption=test_string,
            caption_entities=entities,
        )

        assert message.caption == test_string
        assert message.caption_entities == tuple(entities)

    @pytest.mark.parametrize("default_bot", [{"parse_mode": "Markdown"}], indirect=True)
    async def test_send_live_photo_default_parse_mode_1(
        self, default_bot, chat_id, video_file, photo_file
    ):
        test_string = "Italic Bold Code"
        test_markdown_string = "_Italic_ *Bold* `Code`"

        message = await default_bot.send_live_photo(
            chat_id, photo=photo_file, live_photo=video_file, caption=test_markdown_string
        )
        assert message.caption_markdown == test_markdown_string
        assert message.caption == test_string

    @pytest.mark.parametrize("default_bot", [{"parse_mode": "Markdown"}], indirect=True)
    async def test_send_live_photo_default_parse_mode_2(
        self, default_bot, chat_id, video_file, photo_file
    ):
        test_markdown_string = "_Italic_ *Bold* `Code`"

        message = await default_bot.send_live_photo(
            chat_id,
            photo=photo_file,
            live_photo=video_file,
            caption=test_markdown_string,
            parse_mode=None,
        )
        assert message.caption == test_markdown_string
        assert message.caption_markdown == escape_markdown(test_markdown_string)

    @pytest.mark.parametrize("default_bot", [{"parse_mode": "Markdown"}], indirect=True)
    async def test_send_live_photo_default_parse_mode_3(
        self, default_bot, chat_id, video_file, photo_file
    ):
        test_markdown_string = "_Italic_ *Bold* `Code`"

        message = await default_bot.send_live_photo(
            chat_id,
            photo=photo_file,
            live_photo=video_file,
            caption=test_markdown_string,
            parse_mode="HTML",
        )
        assert message.caption == test_markdown_string
        assert message.caption_markdown == escape_markdown(test_markdown_string)

    @pytest.mark.parametrize(
        ("default_bot", "custom"),
        [
            ({"allow_sending_without_reply": True}, None),
            ({"allow_sending_without_reply": False}, None),
            ({"allow_sending_without_reply": False}, True),
        ],
        indirect=["default_bot"],
    )
    async def test_send_live_photo_default_allow_sending_without_reply(
        self, default_bot, chat_id, video_file, photo_file, custom
    ):
        reply_to_message = await default_bot.send_message(chat_id, "test")
        await reply_to_message.delete()
        if custom is not None:
            message = await default_bot.send_live_photo(
                chat_id,
                photo=photo_file,
                live_photo=video_file,
                allow_sending_without_reply=custom,
                reply_to_message_id=reply_to_message.message_id,
            )
            assert message.reply_to_message is None
        elif default_bot.defaults.allow_sending_without_reply:
            message = await default_bot.send_live_photo(
                chat_id,
                photo=photo_file,
                live_photo=video_file,
                reply_to_message_id=reply_to_message.message_id,
            )
            assert message.reply_to_message is None
        else:
            with pytest.raises(BadRequest, match="Message to be replied not found"):
                await default_bot.send_live_photo(
                    chat_id,
                    photo=photo_file,
                    live_photo=video_file,
                    reply_to_message_id=reply_to_message.message_id,
                )



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram PreparedKeyboardButton."""

from telegram._telegramobject import TelegramObject
from telegram._utils.types import (
    JSONDict,
)


class PreparedKeyboardButton(TelegramObject):
    """
    Describes a keyboard button to be used by a user of a Mini App.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`id` is equal.

    .. versionadded:: 22.8

    Args:
        id (:obj:`str`): Unique identifier of the keyboard button.

    Attributes:
        id (:obj:`str`): Unique identifier of the keyboard button.
    """

    __slots__ = ("id",)

    def __init__(
        self,
        id: str,  # pylint: disable=redefined-builtin
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.id: str = id
        self._id_attrs = (self.id,)
        self._freeze()



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram Bot Access Settings."""

import pytest

from telegram import BotAccessSettings, Dice
from telegram._user import User
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def bot_access_settings():
    return BotAccessSettings(
        is_access_restricted=BotAccessSettingsTestBase.is_access_restricted,
        added_users=BotAccessSettingsTestBase.added_users,
    )


class BotAccessSettingsTestBase:
    is_access_restricted = True
    added_users = [User(id=123, first_name="John", is_bot=False)]


class TestBotAccessSettingsWithoutRequest(BotAccessSettingsTestBase):
    def test_slot_behaviour(self, bot_access_settings):
        inst = bot_access_settings
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "is_access_restricted": self.is_access_restricted,
            "added_users": [user.to_dict() for user in self.added_users],
        }
        bot_access_settings = BotAccessSettings.de_json(json_dict, offline_bot)
        assert bot_access_settings.api_kwargs == {}

        assert bot_access_settings.is_access_restricted == self.is_access_restricted
        assert bot_access_settings.added_users == tuple(self.added_users)

    def test_to_dict(self, bot_access_settings):
        bot_access_settings_dict = bot_access_settings.to_dict()

        assert isinstance(bot_access_settings_dict, dict)
        assert (
            bot_access_settings_dict["is_access_restricted"]
            == bot_access_settings.is_access_restricted
        )
        assert isinstance(bot_access_settings_dict["added_users"], list)
        assert bot_access_settings_dict["added_users"][0] == self.added_users[0].to_dict()

    def test_equality(self):
        a = BotAccessSettings(is_access_restricted=True, added_users=self.added_users)
        b = BotAccessSettings(is_access_restricted=True, added_users=self.added_users)
        c = BotAccessSettings(is_access_restricted=False, added_users=self.added_users)
        d = BotAccessSettings(is_access_restricted=True, added_users=None)
        e = Dice(emoji="🎲", value=4)

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import PreparedKeyboardButton
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def prepared_keyboard_button():
    return PreparedKeyboardButton(PreparedKeyboardButtonTestBase.id)


class PreparedKeyboardButtonTestBase:
    id = "4"


class TestPreparedKeyboardButtonWithoutRequest(PreparedKeyboardButtonTestBase):
    def test_slot_behaviour(self, prepared_keyboard_button):
        inst = prepared_keyboard_button
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "id": self.id,
        }
        prepared_keyboard_button = PreparedKeyboardButton.de_json(json_dict, offline_bot)
        assert prepared_keyboard_button.api_kwargs == {}
        assert prepared_keyboard_button.id == self.id

    def test_to_dict(self, prepared_keyboard_button):
        prepared_keyboard_button_dict = prepared_keyboard_button.to_dict()
        assert prepared_keyboard_button_dict["id"] == self.id

    def test_equality(self):
        a = PreparedKeyboardButton(self.id)
        b = PreparedKeyboardButton(self.id)
        c = PreparedKeyboardButton("5")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)



#!/usr/bin/env python
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import datetime as dtm

import pytest

from telegram import (
    Animation,
    Audio,
    Chat,
    Document,
    InputMediaPhoto,
    InputPollOption,
    LivePhoto,
    Location,
    MaybeInaccessibleMessage,
    MessageEntity,
    PhotoSize,
    Poll,
    PollAnswer,
    PollMedia,
    PollOption,
    PollOptionAdded,
    PollOptionDeleted,
    Sticker,
    User,
    Venue,
    Video,
)
from telegram._utils.datetime import UTC, to_timestamp
from telegram.constants import PollType
from telegram.warnings import PTBDeprecationWarning
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def input_poll_option():
    out = InputPollOption(
        text=InputPollOptionTestBase.text,
        text_parse_mode=InputPollOptionTestBase.text_parse_mode,
        text_entities=InputPollOptionTestBase.text_entities,
        media=InputPollOptionTestBase.media,
    )
    out._unfreeze()
    return out


class InputPollOptionTestBase:
    text = "test option"
    text_parse_mode = "MarkdownV2"
    text_entities = [
        MessageEntity(0, 4, MessageEntity.BOLD),
        MessageEntity(5, 7, MessageEntity.ITALIC),
    ]
    media = InputMediaPhoto("media")


class TestInputPollOptionWithoutRequest(InputPollOptionTestBase):
    def test_slot_behaviour(self, input_poll_option):
        for attr in input_poll_option.__slots__:
            assert getattr(input_poll_option, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(input_poll_option)) == len(set(mro_slots(input_poll_option))), (
            "duplicate slot"
        )

    # tags: deprecated NEXT.VERSION
    def test_de_json(self):
        json_dict = {
            "text": self.text,
            "text_parse_mode": self.text_parse_mode,
            "text_entities": [e.to_dict() for e in self.text_entities],
        }
        input_poll_option = InputPollOption.de_json(json_dict, None)
        assert input_poll_option.api_kwargs == {}

        assert input_poll_option.text == self.text
        assert input_poll_option.text_parse_mode == self.text_parse_mode
        assert input_poll_option.text_entities == tuple(self.text_entities)

    def test_de_json_deprecated(self, recwarn):
        InputPollOption.de_json({"text": self.text}, None)

        assert len(recwarn) == 1
        assert "`InputPollOption.de_json` is deprecated" in str(recwarn[0].message)
        assert "The `media` field will not be included for deserialization" in str(
            recwarn[0].message
        )
        assert recwarn[0].category is PTBDeprecationWarning

    def test_to_dict(self, input_poll_option):
        input_poll_option_dict = input_poll_option.to_dict()

        assert isinstance(input_poll_option_dict, dict)
        assert input_poll_option_dict["text"] == input_poll_option.text
        assert input_poll_option_dict["text_parse_mode"] == input_poll_option.text_parse_mode
        assert input_poll_option_dict["text_entities"] == [
            e.to_dict() for e in input_poll_option.text_entities
        ]
        assert input_poll_option_dict["media"] == input_poll_option.media.to_dict()

        # Test that the default-value parameter is handled correctly
        input_poll_option = InputPollOption("text")
        input_poll_option_dict = input_poll_option.to_dict()
        assert "text_parse_mode" not in input_poll_option_dict

    def test_equality(self):
        a = InputPollOption("text")
        b = InputPollOption("text", self.text_parse_mode)
        c = InputPollOption("text", text_entities=self.text_entities)
        d = InputPollOption("different_text")
        e = Poll(
            123,
            "question",
            ["O1", "O2"],
            1,
            False,
            True,
            Poll.REGULAR,
            True,
            allows_revoting=True,
            members_only=True,
        )

        assert a == b
        assert hash(a) == hash(b)

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture(scope="module")
def poll_media():
    return PollMedia(
        animation=PollMediaTestBase.animation,
        audio=PollMediaTestBase.audio,
        document=PollMediaTestBase.document,
        live_photo=PollMediaTestBase.live_photo,
        location=PollMediaTestBase.location,
        photo=PollMediaTestBase.photo,
        sticker=PollMediaTestBase.sticker,
        venue=PollMediaTestBase.venue,
        video=PollMediaTestBase.video,
    )


class PollMediaTestBase:
    animation = Animation("blah", "unique_id", 320, 180, 1)
    audio = Audio(file_id="file_id", file_unique_id="file_unique_id", duration=30)
    document = Document("file_id", "file_unique_id", "file_name", 42)
    location = Location(123, 456)
    photo = (PhotoSize("file_id", "file_unique_id", 1, 1),)
    sticker = Sticker("file_id", "file_unique_id", 512, 512, False, False, "regular")
    venue = Venue(location=Location(123, 456), title="title", address="address")
    video = Video(
        file_id="video_file_id",
        width=640,
        height=480,
        file_unique_id="file_unique_id",
        duration=dtm.timedelta(seconds=60),
    )
    live_photo = LivePhoto(
        file_id="video_file_id",
        file_unique_id="file_unique_id",
        width=640,
        height=480,
        duration=dtm.timedelta(seconds=60),
        mime_type="video/mp4",
        file_size=326534,
    )


class TestPollMediaWithoutRequest(PollMediaTestBase):
    def test_slot_behaviour(self, poll_media):
        for attr in poll_media.__slots__:
            assert getattr(poll_media, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(poll_media)) == len(set(mro_slots(poll_media))), "duplicate slot"

    def test_de_json(self):
        json_dict = {
            "animation": self.animation.to_dict(),
            "audio": self.audio.to_dict(),
            "document": self.document.to_dict(),
            "live_photo": self.live_photo.to_dict(),
            "location": self.location.to_dict(),
            "photo": [photo.to_dict() for photo in self.photo],
            "sticker": self.sticker.to_dict(),
            "venue": self.venue.to_dict(),
            "video": self.video.to_dict(),
        }
        poll_media = PollMedia.de_json(json_dict, None)

        assert poll_media.api_kwargs == {}
        assert poll_media.animation == self.animation
        assert poll_media.audio == self.audio
        assert poll_media.document == self.document
        assert poll_media.live_photo == self.live_photo
        assert poll_media.location == self.location
        assert poll_media.photo == self.photo
        assert poll_media.sticker == self.sticker
        assert poll_media.venue == self.venue
        assert poll_media.video == self.video

    def test_to_dict(self, poll_media):
        poll_media_dict = poll_media.to_dict()

        assert isinstance(poll_media_dict, dict)
        assert poll_media_dict["animation"] == poll_media.animation.to_dict()
        assert poll_media_dict["audio"] == poll_media.audio.to_dict()
        assert poll_media_dict["document"] == poll_media.document.to_dict()
        assert poll_media_dict["live_photo"] == poll_media.live_photo.to_dict()
        assert poll_media_dict["location"] == poll_media.location.to_dict()
        assert poll_media_dict["photo"] == [photo.to_dict() for photo in poll_media.photo]
        assert poll_media_dict["sticker"] == poll_media.sticker.to_dict()
        assert poll_media_dict["venue"] == poll_media.venue.to_dict()
        assert poll_media_dict["video"] == poll_media.video.to_dict()

    def test_equality(self):
        a = PollMedia(photo=self.photo)
        b = PollMedia(photo=self.photo)
        c = PollMedia(photo=(PhotoSize("file_id", "other_file_unique_id", 1, 1),))
        d = PollMedia(video=self.video)
        e = PollOption("text", 1, persistent_id="persistent_id")

        assert a == b
        assert hash(a) == hash(b)

        assert a != d
        assert hash(a) != hash(d)

        assert a != c
        assert hash(a) != hash(c)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture(scope="module")
def poll_option():
    out = PollOption(
        text=PollOptionTestBase.text,
        voter_count=PollOptionTestBase.voter_count,
        text_entities=PollOptionTestBase.text_entities,
        added_by_user=PollOptionTestBase.added_by_user,
        added_by_chat=PollOptionTestBase.added_by_chat,
        addition_date=PollOptionTestBase.addition_date,
        persistent_id=PollOptionTestBase.persistent_id,
        media=PollOptionTestBase.media,
    )
    out._unfreeze()
    return out


class PollOptionTestBase:
    text = "test option"
    voter_count = 3
    text_entities = [
        MessageEntity(MessageEntity.BOLD, 0, 4),
        MessageEntity(MessageEntity.ITALIC, 5, 6),
    ]
    added_by_user = User(1, "test_user", False)
    added_by_chat = Chat(1, "test_chat")
    addition_date = dtm.datetime.now(dtm.timezone.utc)
    persistent_id = "persistent_id"
    media = PollMedia(location=Location(123, 456))


class TestPollOptionWithoutRequest(PollOptionTestBase):
    def test_slot_behaviour(self, poll_option):
        for attr in poll_option.__slots__:
            assert getattr(poll_option, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(poll_option)) == len(set(mro_slots(poll_option))), "duplicate slot"

    def test_de_json(self):
        json_dict = {
            "text": self.text,
            "voter_count": self.voter_count,
            "text_entities": [e.to_dict() for e in self.text_entities],
            "added_by_user": self.added_by_user.to_dict(),
            "added_by_chat": self.added_by_chat.to_dict(),
            "addition_date": to_timestamp(self.addition_date),
            "persistent_id": self.persistent_id,
            "media": self.media.to_dict(),
        }
        poll_option = PollOption.de_json(json_dict, None)
        assert poll_option.api_kwargs == {}

        assert poll_option.text == self.text
        assert poll_option.voter_count == self.voter_count
        assert poll_option.text_entities == tuple(self.text_entities)
        assert poll_option.added_by_user == self.added_by_user
        assert poll_option.added_by_chat == self.added_by_chat
        assert abs((poll_option.addition_date - self.addition_date).total_seconds()) < 1
        assert poll_option.persistent_id == self.persistent_id
        assert poll_option.media == self.media

    def test_to_dict(self, poll_option):
        poll_option_dict = poll_option.to_dict()

        assert isinstance(poll_option_dict, dict)
        assert poll_option_dict["text"] == poll_option.text
        assert poll_option_dict["voter_count"] == poll_option.voter_count
        assert poll_option_dict["text_entities"] == [
            e.to_dict() for e in poll_option.text_entities
        ]
        assert poll_option_dict["added_by_user"] == poll_option.added_by_user.to_dict()
        assert poll_option_dict["added_by_chat"] == poll_option.added_by_chat.to_dict()
        assert poll_option_dict["addition_date"] == to_timestamp(poll_option.addition_date)
        assert poll_option_dict["persistent_id"] == poll_option.persistent_id
        assert poll_option_dict["media"] == poll_option.media.to_dict()

    def test_parse_entity(self, poll_option):
        entity = MessageEntity(MessageEntity.BOLD, 0, 4)
        poll_option.text_entities = [entity]

        assert poll_option.parse_entity(entity) == "test"

    def test_parse_entities(self, poll_option):
        entity = MessageEntity(MessageEntity.BOLD, 0, 4)
        entity_2 = MessageEntity(MessageEntity.ITALIC, 5, 6)
        poll_option.text_entities = [entity, entity_2]

        assert poll_option.parse_entities(MessageEntity.BOLD) == {entity: "test"}
        assert poll_option.parse_entities() == {entity: "test", entity_2: "option"}

    def test_persistent_id_required_workaround(self):
        # tags: deprecated NEXT.VERSION, bot api 9.6
        with pytest.raises(TypeError, match="`persistent_id` is a required"):
            PollOption(self.text, self.voter_count)

    def test_equality(self):
        a = PollOption("text", 1, persistent_id="persistent_id")
        b = PollOption("text", 1, persistent_id="persistent_id")
        c = PollOption("other_text", 1, persistent_id="persistent_id")
        d = PollOption("text", 1 + 9, persistent_id="persistent_id")
        e = PollOption("text", 1, persistent_id="other_persistent_id")
        f = Poll(
            123,
            "question",
            ["O1", "O2"],
            1,
            False,
            True,
            Poll.REGULAR,
            True,
            allows_revoting=True,
            members_only=True,
        )

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

        assert a != f
        assert hash(a) != hash(f)


@pytest.fixture(scope="module")
def poll_answer():
    return PollAnswer(
        PollAnswerTestBase.poll_id,
        PollAnswerTestBase.option_ids,
        PollAnswerTestBase.user,
        PollAnswerTestBase.voter_chat,
        PollAnswerTestBase.option_persistent_ids,
    )


class PollAnswerTestBase:
    poll_id = "id"
    option_ids = [2]
    user = User(1, "", False)
    voter_chat = Chat(1, "")
    option_persistent_ids = ["123"]


class TestPollAnswerWithoutRequest(PollAnswerTestBase):
    def test_de_json(self):
        json_dict = {
            "poll_id": self.poll_id,
            "option_ids": self.option_ids,
            "user": self.user.to_dict(),
            "voter_chat": self.voter_chat.to_dict(),
            "option_persistent_ids": self.option_persistent_ids,
        }
        poll_answer = PollAnswer.de_json(json_dict, None)
        assert poll_answer.api_kwargs == {}

        assert poll_answer.poll_id == self.poll_id
        assert poll_answer.option_ids == tuple(self.option_ids)
        assert poll_answer.user == self.user
        assert poll_answer.voter_chat == self.voter_chat
        assert poll_answer.option_persistent_ids == tuple(self.option_persistent_ids)

    def test_to_dict(self, poll_answer):
        poll_answer_dict = poll_answer.to_dict()

        assert isinstance(poll_answer_dict, dict)
        assert poll_answer_dict["poll_id"] == poll_answer.poll_id
        assert poll_answer_dict["option_ids"] == list(poll_answer.option_ids)
        assert poll_answer_dict["user"] == poll_answer.user.to_dict()
        assert poll_answer_dict["voter_chat"] == poll_answer.voter_chat.to_dict()
        assert poll_answer_dict["option_persistent_ids"] == list(poll_answer.option_persistent_ids)

    def test_persistent_id_required_workaround(self):
        # tags: deprecated NEXT.VERSION, bot api 9.6
        with pytest.raises(TypeError, match="`option_persistent_ids` is a required"):
            PollAnswer(poll_id=123, option_ids=[2], user=self.user, voter_chat=self.voter_chat)

    def test_equality(self):
        a = PollAnswer(123, [2], self.user, self.voter_chat, option_persistent_ids=["2"])
        b = PollAnswer(123, [2], self.user, Chat(1, ""), option_persistent_ids=["2"])
        c = PollAnswer(
            123, [2], User(1, "first", False), self.voter_chat, option_persistent_ids=["2"]
        )
        d = PollAnswer(123, [1, 2], self.user, self.voter_chat, option_persistent_ids=["1", "2"])
        e = PollAnswer(456, [2], self.user, self.voter_chat, option_persistent_ids=["2"])
        f = PollOption("Text", 1, persistent_id="persistent_id")

        assert a == b
        assert hash(a) == hash(b)

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

        assert a != f
        assert hash(a) != hash(f)


@pytest.fixture(scope="module")
def poll():
    poll = Poll(
        PollTestBase.id_,
        PollTestBase.question,
        PollTestBase.options,
        PollTestBase.total_voter_count,
        PollTestBase.is_closed,
        PollTestBase.is_anonymous,
        PollTestBase.type,
        PollTestBase.allows_multiple_answers,
        explanation=PollTestBase.explanation,
        explanation_entities=PollTestBase.explanation_entities,
        open_period=PollTestBase.open_period,
        close_date=PollTestBase.close_date,
        question_entities=PollTestBase.question_entities,
        allows_revoting=PollTestBase.allows_revoting,
        members_only=PollTestBase.members_only,
        correct_option_ids=PollTestBase.correct_option_ids,
        description=PollTestBase.description,
        description_entities=PollTestBase.description_entities,
        country_codes=PollTestBase.country_codes,
        media=PollTestBase.media,
        explanation_media=PollTestBase.explanation_media,
    )
    poll._unfreeze()
    return poll


class PollTestBase:
    id_ = "id"
    question = "Test Question?"
    options = [
        PollOption("test", 10, persistent_id="persistent_id"),
        PollOption("test2", 11, persistent_id="persistent_id_2"),
    ]
    total_voter_count = 0
    is_closed = True
    is_anonymous = False
    type = Poll.REGULAR
    allows_multiple_answers = True
    members_only = True
    explanation = (
        b"\\U0001f469\\u200d\\U0001f469\\u200d\\U0001f467"
        b"\\u200d\\U0001f467\\U0001f431http://google.com"
    ).decode("unicode-escape")
    explanation_entities = [MessageEntity(13, 17, MessageEntity.URL)]
    open_period = dtm.timedelta(seconds=42)
    close_date = dtm.datetime.now(dtm.timezone.utc)
    question_entities = [
        MessageEntity(MessageEntity.BOLD, 0, 4),
        MessageEntity(MessageEntity.ITALIC, 5, 8),
    ]
    allows_revoting = True
    correct_option_ids = [1, 2]
    description = "description"
    description_entities = [MessageEntity(MessageEntity.ITALIC, 0, 11)]
    country_codes = ["AB", "CD"]
    media = PollMedia(document=Document("file_id", "file_unique_id", "file_name", 42))
    explanation_media = PollMedia(animation=Animation("blah", "unique_id", 320, 180, 1))


class TestPollWithoutRequest(PollTestBase):
    def test_de_json(self, offline_bot):
        json_dict = {
            "id": self.id_,
            "question": self.question,
            "options": [o.to_dict() for o in self.options],
            "total_voter_count": self.total_voter_count,
            "is_closed": self.is_closed,
            "is_anonymous": self.is_anonymous,
            "type": self.type,
            "allows_multiple_answers": self.allows_multiple_answers,
            "explanation": self.explanation,
            "explanation_entities": [self.explanation_entities[0].to_dict()],
            "open_period": int(self.open_period.total_seconds()),
            "close_date": to_timestamp(self.close_date),
            "question_entities": [e.to_dict() for e in self.question_entities],
            "allows_revoting": self.allows_revoting,
            "members_only": self.members_only,
            "correct_option_ids": self.correct_option_ids,
            "description": self.description,
            "description_entities": [e.to_dict() for e in self.description_entities],
            "country_codes": self.country_codes,
            "media": self.media.to_dict(),
            "explanation_media": self.explanation_media.to_dict(),
        }
        poll = Poll.de_json(json_dict, offline_bot)
        assert poll.api_kwargs == {}

        assert poll.id == self.id_
        assert poll.question == self.question
        assert poll.options == tuple(self.options)
        assert poll.options[0].text == self.options[0].text
        assert poll.options[0].voter_count == self.options[0].voter_count
        assert poll.options[1].text == self.options[1].text
        assert poll.options[1].voter_count == self.options[1].voter_count
        assert poll.total_voter_count == self.total_voter_count
        assert poll.is_closed == self.is_closed
        assert poll.is_anonymous == self.is_anonymous
        assert poll.type == self.type
        assert poll.allows_multiple_answers == self.allows_multiple_answers
        assert poll.members_only == self.members_only
        assert poll.explanation == self.explanation
        assert poll.explanation_entities == tuple(self.explanation_entities)
        assert poll._open_period == self.open_period
        assert abs(poll.close_date - self.close_date) < dtm.timedelta(seconds=1)
        assert to_timestamp(poll.close_date) == to_timestamp(self.close_date)
        assert poll.question_entities == tuple(self.question_entities)
        assert poll.allows_revoting == self.allows_revoting
        assert poll.correct_option_ids == tuple(self.correct_option_ids)
        assert poll.description == self.description
        assert poll.description_entities == tuple(self.description_entities)
        assert poll.country_codes == tuple(self.country_codes)
        assert poll.media == self.media
        assert poll.explanation_media == self.explanation_media

    def test_de_json_localization(self, tz_bot, offline_bot, raw_bot):
        json_dict = {
            "id": self.id_,
            "question": self.question,
            "options": [o.to_dict() for o in self.options],
            "total_voter_count": self.total_voter_count,
            "is_closed": self.is_closed,
            "is_anonymous": self.is_anonymous,
            "type": self.type,
            "allows_multiple_answers": self.allows_multiple_answers,
            "explanation": self.explanation,
            "explanation_entities": [self.explanation_entities[0].to_dict()],
            "open_period": int(self.open_period.total_seconds()),
            "close_date": to_timestamp(self.close_date),
            "question_entities": [e.to_dict() for e in self.question_entities],
            "allows_revoting": self.allows_revoting,
            "members_only": self.members_only,
            "correct_option_ids": self.correct_option_ids,
            "description": self.description,
            "description_entities": [e.to_dict() for e in self.description_entities],
            "country_codes": self.country_codes,
            "media": self.media.to_dict(),
            "explanation_media": self.explanation_media.to_dict(),
        }

        poll_raw = Poll.de_json(json_dict, raw_bot)
        poll_bot = Poll.de_json(json_dict, offline_bot)
        poll_bot_tz = Poll.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing timezones is unpredicatable
        poll_bot_tz_offset = poll_bot_tz.close_date.utcoffset()
        tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(
            poll_bot_tz.close_date.replace(tzinfo=None)
        )

        assert poll_raw.close_date.tzinfo == UTC
        assert poll_bot.close_date.tzinfo == UTC
        assert poll_bot_tz_offset == tz_bot_offset

    def test_to_dict(self, poll):
        poll_dict = poll.to_dict()

        assert isinstance(poll_dict, dict)
        assert poll_dict["id"] == poll.id
        assert poll_dict["question"] == poll.question
        assert poll_dict["options"] == [o.to_dict() for o in poll.options]
        assert poll_dict["total_voter_count"] == poll.total_voter_count
        assert poll_dict["is_closed"] == poll.is_closed
        assert poll_dict["is_anonymous"] == poll.is_anonymous
        assert poll_dict["type"] == poll.type
        assert poll_dict["allows_multiple_answers"] == poll.allows_multiple_answers
        assert poll_dict["members_only"] == poll.members_only
        assert poll_dict["explanation"] == poll.explanation
        assert poll_dict["explanation_entities"] == [poll.explanation_entities[0].to_dict()]
        assert poll_dict["open_period"] == int(self.open_period.total_seconds())
        assert poll_dict["close_date"] == to_timestamp(poll.close_date)
        assert poll_dict["question_entities"] == [e.to_dict() for e in poll.question_entities]
        assert poll_dict["allows_revoting"] == poll.allows_revoting
        assert poll_dict["correct_option_ids"] == list(poll.correct_option_ids)
        assert poll_dict["description"] == poll.description
        assert poll_dict["description_entities"] == [
            e.to_dict() for e in poll.description_entities
        ]
        assert poll_dict["country_codes"] == list(poll.country_codes)
        assert poll_dict["media"] == poll.media.to_dict()
        assert poll_dict["explanation_media"] == poll.explanation_media.to_dict()

    def test_time_period_properties(self, PTB_TIMEDELTA, poll):
        if PTB_TIMEDELTA:
            assert poll.open_period == self.open_period
            assert isinstance(poll.open_period, dtm.timedelta)
        else:
            assert poll.open_period == int(self.open_period.total_seconds())
            assert isinstance(poll.open_period, int)

    def test_time_period_int_deprecated(self, recwarn, PTB_TIMEDELTA, poll):
        poll.open_period

        if PTB_TIMEDELTA:
            assert len(recwarn) == 0
        else:
            assert len(recwarn) == 1
            assert "`open_period` will be of type `datetime.timedelta`" in str(recwarn[0].message)
            assert recwarn[0].category is PTBDeprecationWarning

    def test_correct_option_id_deprecated(self, recwarn, poll):
        poll.correct_option_id

        assert len(recwarn) == 1
        assert "The attribute `correct_option_id` is deprecated" in str(recwarn[0].message)
        assert recwarn[0].category is PTBDeprecationWarning

        poll = Poll(
            PollTestBase.id_,
            PollTestBase.question,
            PollTestBase.options,
            PollTestBase.total_voter_count,
            PollTestBase.is_closed,
            PollTestBase.is_anonymous,
            PollTestBase.type,
            PollTestBase.allows_multiple_answers,
            correct_option_id=1,
            allows_revoting=PollTestBase.allows_revoting,
            members_only=PollTestBase.members_only,
        )
        assert poll.correct_option_ids == (1,)

    def test_allows_revoting_required_workaround(self):
        # tags: deprecated NEXT.VERSION, bot api 9.6
        with pytest.raises(TypeError, match="`allows_revoting` is a required"):
            Poll(
                self.id_,
                self.question,
                self.options,
                self.total_voter_count,
                self.is_closed,
                self.is_anonymous,
                self.type,
                self.allows_multiple_answers,
                members_only=self.members_only,
            )

    def test_members_only_required_workaround(self):
        # tags: deprecated NEXT.VERSION, bot api 10.0
        with pytest.raises(TypeError, match="`members_only` is a required"):
            Poll(
                self.id_,
                self.question,
                self.options,
                self.total_voter_count,
                self.is_closed,
                self.is_anonymous,
                self.type,
                self.allows_multiple_answers,
                allows_revoting=self.allows_revoting,
            )

    def test_equality(self):
        a = Poll(
            123,
            "question",
            ["O1", "O2"],
            1,
            False,
            True,
            Poll.REGULAR,
            True,
            allows_revoting=True,
            members_only=True,
        )
        b = Poll(
            123,
            "question",
            ["o1", "o2"],
            1,
            True,
            False,
            Poll.REGULAR,
            True,
            allows_revoting=False,
            members_only=False,
        )
        c = Poll(
            456,
            "question",
            ["o1", "o2"],
            1,
            True,
            False,
            Poll.REGULAR,
            True,
            allows_revoting=True,
            members_only=True,
        )
        d = PollOption("Text", 1, persistent_id="persistent_id")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

    def test_enum_init(self):
        poll = Poll(
            type="foo",
            id="id",
            question="question",
            options=[],
            total_voter_count=0,
            is_closed=False,
            is_anonymous=False,
            allows_multiple_answers=False,
            allows_revoting=True,
            members_only=True,
        )
        assert poll.type == "foo"
        poll = Poll(
            type=PollType.QUIZ,
            id="id",
            question="question",
            options=[],
            total_voter_count=0,
            is_closed=False,
            is_anonymous=False,
            allows_multiple_answers=False,
            allows_revoting=True,
            members_only=True,
        )
        assert poll.type is PollType.QUIZ

    def test_parse_explanation_entity(self, poll):
        entity = MessageEntity(type=MessageEntity.URL, offset=13, length=17)
        poll.explanation_entities = [entity]

        assert poll.parse_explanation_entity(entity) == "http://google.com"

        with pytest.raises(RuntimeError, match="Poll has no"):
            Poll(
                "id",
                "question",
                [PollOption("text", voter_count=0, persistent_id="persistent_id")],
                total_voter_count=0,
                is_closed=False,
                is_anonymous=False,
                type=Poll.QUIZ,
                allows_multiple_answers=False,
                allows_revoting=True,
                members_only=True,
            ).parse_explanation_entity(entity)

    def test_parse_explanation_entities(self, poll):
        entity = MessageEntity(type=MessageEntity.URL, offset=13, length=17)
        entity_2 = MessageEntity(type=MessageEntity.BOLD, offset=13, length=1)
        poll.explanation_entities = [entity_2, entity]

        assert poll.parse_explanation_entities(MessageEntity.URL) == {entity: "http://google.com"}
        assert poll.parse_explanation_entities() == {entity: "http://google.com", entity_2: "h"}

        with pytest.raises(RuntimeError, match="Poll has no"):
            Poll(
                "id",
                "question",
                [PollOption("text", voter_count=0, persistent_id="persistent_id")],
                total_voter_count=0,
                is_closed=False,
                is_anonymous=False,
                type=Poll.QUIZ,
                allows_multiple_answers=False,
                allows_revoting=True,
                members_only=True,
            ).parse_explanation_entities()

    def test_parse_question_entity(self, poll):
        entity = MessageEntity(MessageEntity.ITALIC, 5, 8)
        poll.question_entities = [entity]

        assert poll.parse_question_entity(entity) == "Question"

    def test_parse_question_entities(self, poll):
        entity = MessageEntity(MessageEntity.ITALIC, 5, 8)
        entity_2 = MessageEntity(MessageEntity.BOLD, 0, 4)
        poll.question_entities = [entity_2, entity]

        assert poll.parse_question_entities(MessageEntity.ITALIC) == {entity: "Question"}
        assert poll.parse_question_entities() == {entity: "Question", entity_2: "Test"}

    def test_parse_description_entity(self, poll):
        entity = MessageEntity(MessageEntity.ITALIC, 0, 11)
        poll.description_entities = [entity]

        assert poll.parse_description_entity(entity) == "description"
        with pytest.raises(RuntimeError, match="Poll has no"):
            Poll(
                "id",
                "question",
                [PollOption("text", voter_count=0, persistent_id="persistent_id")],
                total_voter_count=0,
                is_closed=False,
                is_anonymous=False,
                type=Poll.QUIZ,
                allows_multiple_answers=False,
                allows_revoting=True,
                members_only=True,
            ).parse_description_entity(entity)

    def test_parse_description_entities(self, poll):
        entity = MessageEntity(MessageEntity.ITALIC, 0, 11)
        entity_2 = MessageEntity(MessageEntity.BOLD, 0, 4)
        poll.description_entities = [entity_2, entity]

        assert poll.parse_description_entities(MessageEntity.ITALIC) == {entity: "description"}
        assert poll.parse_description_entities() == {entity: "description", entity_2: "desc"}
        with pytest.raises(RuntimeError, match="Poll has no"):
            Poll(
                "id",
                "question",
                [PollOption("text", voter_count=0, persistent_id="persistent_id")],
                total_voter_count=0,
                is_closed=False,
                is_anonymous=False,
                type=Poll.QUIZ,
                allows_multiple_answers=False,
                allows_revoting=True,
                members_only=True,
            ).parse_description_entities()


@pytest.fixture(scope="module")
def poll_option_added():
    p = PollOptionAdded(
        poll_message=PollOptionAddedTestBase.poll_message,
        option_persistent_id=PollOptionAddedTestBase.option_persistent_id,
        option_text=PollOptionAddedTestBase.option_text,
        option_text_entities=PollOptionAddedTestBase.option_text_entities,
    )
    p._unfreeze()
    return p


class PollOptionAddedTestBase:
    poll_message = MaybeInaccessibleMessage(
        message_id=1,
        date=dtm.datetime.now(dtm.timezone.utc),
        chat=Chat(1, "test_chat"),
    )
    option_persistent_id = "123"
    option_text = "test option"
    option_text_entities = [
        MessageEntity(MessageEntity.BOLD, 0, 4),
        MessageEntity(MessageEntity.ITALIC, 5, 6),
    ]


class TestPollOptionAddedWithoutRequest(PollOptionAddedTestBase):
    def test_slot_behaviour(self, poll_option_added):
        for attr in poll_option_added.__slots__:
            assert getattr(poll_option_added, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(poll_option_added)) == len(set(mro_slots(poll_option_added))), (
            "duplicate slot"
        )

    def test_de_json(self, offline_bot):
        json_dict = {
            "poll_message": self.poll_message.to_dict(),
            "option_persistent_id": self.option_persistent_id,
            "option_text": self.option_text,
            "option_text_entities": [e.to_dict() for e in self.option_text_entities],
        }
        poll_option_added = PollOptionAdded.de_json(json_dict, offline_bot)
        assert poll_option_added.api_kwargs == {}

        assert poll_option_added.poll_message == self.poll_message
        assert poll_option_added.option_persistent_id == self.option_persistent_id
        assert poll_option_added.option_text == self.option_text
        assert poll_option_added.option_text_entities == tuple(self.option_text_entities)

    def test_to_dict(self, poll_option_added):
        poll_option_added_dict = poll_option_added.to_dict()

        assert isinstance(poll_option_added_dict, dict)
        assert poll_option_added_dict["poll_message"] == poll_option_added.poll_message.to_dict()
        assert (
            poll_option_added_dict["option_persistent_id"]
            == poll_option_added.option_persistent_id
        )
        assert poll_option_added_dict["option_text"] == poll_option_added.option_text
        assert poll_option_added_dict["option_text_entities"] == [
            e.to_dict() for e in poll_option_added.option_text_entities
        ]

    def test_parse_option_text_entity(self, poll_option_added):
        entity = MessageEntity(MessageEntity.BOLD, 0, 4)
        poll_option_added.option_text_entities = [entity]

        assert poll_option_added.parse_option_text_entity(entity) == "test"

    def test_parse_option_text_entities(self, poll_option_added):
        entity = MessageEntity(MessageEntity.BOLD, 0, 4)
        entity_2 = MessageEntity(MessageEntity.ITALIC, 5, 6)
        poll_option_added.option_text_entities = [entity, entity_2]

        assert poll_option_added.parse_option_text_entities(MessageEntity.BOLD) == {entity: "test"}
        assert poll_option_added.parse_option_text_entities() == {
            entity: "test",
            entity_2: "option",
        }

    def test_equality(self):
        a = PollOptionAdded(
            poll_message=self.poll_message,
            option_persistent_id=self.option_persistent_id,
            option_text=self.option_text,
            option_text_entities=self.option_text_entities,
        )
        b = PollOptionAdded(
            poll_message=self.poll_message,
            option_persistent_id=self.option_persistent_id,
            option_text=self.option_text,
            option_text_entities=self.option_text_entities,
        )
        c = PollOptionAdded(
            poll_message=MaybeInaccessibleMessage(
                2, dtm.datetime.now(dtm.timezone.utc), Chat(1, "test_chat")
            ),
            option_persistent_id=self.option_persistent_id,
            option_text=self.option_text,
            option_text_entities=self.option_text_entities,
        )

        d = PollOptionAdded(
            poll_message=self.poll_message,
            option_persistent_id="different_id",
            option_text=self.option_text,
            option_text_entities=self.option_text_entities,
        )

        assert a == b
        assert hash(a) == hash(b)

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture(scope="module")
def poll_option_deleted():
    p = PollOptionDeleted(
        poll_message=PollOptionDeletedTestBase.poll_message,
        option_persistent_id=PollOptionDeletedTestBase.option_persistent_id,
        option_text=PollOptionDeletedTestBase.option_text,
        option_text_entities=PollOptionDeletedTestBase.option_text_entities,
    )
    p._unfreeze()
    return p


class PollOptionDeletedTestBase:
    poll_message = MaybeInaccessibleMessage(
        message_id=1,
        date=dtm.datetime.now(dtm.timezone.utc),
        chat=Chat(1, "test_chat"),
    )
    option_persistent_id = "123"
    option_text = "test option"
    option_text_entities = [
        MessageEntity(MessageEntity.BOLD, 0, 4),
        MessageEntity(MessageEntity.ITALIC, 5, 6),
    ]


class TestPollOptionDeletedWithoutRequest(PollOptionDeletedTestBase):
    def test_slot_behaviour(self, poll_option_deleted):
        for attr in poll_option_deleted.__slots__:
            assert getattr(poll_option_deleted, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(poll_option_deleted)) == len(set(mro_slots(poll_option_deleted))), (
            "duplicate slot"
        )

    def test_de_json(self, offline_bot):
        json_dict = {
            "poll_message": self.poll_message.to_dict(),
            "option_persistent_id": self.option_persistent_id,
            "option_text": self.option_text,
            "option_text_entities": [e.to_dict() for e in self.option_text_entities],
        }
        poll_option_deleted = PollOptionDeleted.de_json(json_dict, offline_bot)
        assert poll_option_deleted.api_kwargs == {}

        assert poll_option_deleted.poll_message == self.poll_message
        assert poll_option_deleted.option_persistent_id == self.option_persistent_id
        assert poll_option_deleted.option_text == self.option_text
        assert poll_option_deleted.option_text_entities == tuple(self.option_text_entities)

    def test_to_dict(self, poll_option_deleted):
        poll_option_deleted_dict = poll_option_deleted.to_dict()

        assert isinstance(poll_option_deleted_dict, dict)
        assert (
            poll_option_deleted_dict["poll_message"] == poll_option_deleted.poll_message.to_dict()
        )
        assert (
            poll_option_deleted_dict["option_persistent_id"]
            == poll_option_deleted.option_persistent_id
        )
        assert poll_option_deleted_dict["option_text"] == poll_option_deleted.option_text
        assert poll_option_deleted_dict["option_text_entities"] == [
            e.to_dict() for e in poll_option_deleted.option_text_entities
        ]

    def test_parse_option_text_entity(self, poll_option_deleted):
        entity = MessageEntity(MessageEntity.BOLD, 0, 4)
        poll_option_deleted.option_text_entities = [entity]

        assert poll_option_deleted.parse_option_text_entity(entity) == "test"

    def test_parse_option_text_entities(self, poll_option_deleted):
        entity = MessageEntity(MessageEntity.BOLD, 0, 4)
        entity_2 = MessageEntity(MessageEntity.ITALIC, 5, 6)
        poll_option_deleted.option_text_entities = [entity, entity_2]

        assert poll_option_deleted.parse_option_text_entities(MessageEntity.BOLD) == {
            entity: "test"
        }
        assert poll_option_deleted.parse_option_text_entities() == {
            entity: "test",
            entity_2: "option",
        }

    def test_equality(self):
        a = PollOptionDeleted(
            poll_message=self.poll_message,
            option_persistent_id=self.option_persistent_id,
            option_text=self.option_text,
            option_text_entities=self.option_text_entities,
        )
        b = PollOptionDeleted(
            poll_message=self.poll_message,
            option_persistent_id=self.option_persistent_id,
            option_text=self.option_text,
            option_text_entities=self.option_text_entities,
        )
        c = PollOptionDeleted(
            poll_message=MaybeInaccessibleMessage(
                2, dtm.datetime.now(dtm.timezone.utc), Chat(1, "test_chat")
            ),
            option_persistent_id=self.option_persistent_id,
            option_text=self.option_text,
            option_text_entities=self.option_text_entities,
        )

        d = PollOptionDeleted(
            poll_message=self.poll_message,
            option_persistent_id="different_id",
            option_text=self.option_text,
            option_text_entities=self.option_text_entities,
        )

        assert a == b
        assert hash(a) == hash(b)

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import ManagedBotCreated, ManagedBotUpdated, User
from tests.auxil.slots import mro_slots


@pytest.fixture
def managed_bot_created():
    return ManagedBotCreated(
        bot=User(
            id=ManagedBotTestBase.bot_id,
            is_bot=ManagedBotTestBase.is_bot,
            first_name=ManagedBotTestBase.bot_first_name,
            username=ManagedBotTestBase.bot_username,
        ),
    )


@pytest.fixture
def managed_bot_updated():
    return ManagedBotUpdated(
        user=User(
            id=ManagedBotTestBase.bot_creator_id,
            is_bot=ManagedBotTestBase.is_bot,
            first_name=ManagedBotTestBase.bot_creator_first_name,
            username=ManagedBotTestBase.bot_creator_username,
        ),
        bot=User(
            id=ManagedBotTestBase.bot_id,
            is_bot=ManagedBotTestBase.is_bot,
            first_name=ManagedBotTestBase.bot_first_name,
            username=ManagedBotTestBase.bot_username,
        ),
    )


class ManagedBotTestBase:
    bot_creator_id = 123
    bot_creator_first_name = "TestBotManager"
    bot_creator_username = "test_bot_manager"
    bot_id = 321
    bot_first_name = "TestBot"
    bot_username = "test_bot"
    is_bot = True


class TestManagedBotCreatedWithoutRequest(ManagedBotTestBase):
    def test_slot_behaviour(self, managed_bot_created):
        inst = managed_bot_created
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot, managed_bot_created):
        json_dict = {
            "bot": managed_bot_created.bot.to_dict(),
        }
        managed_bot_created = ManagedBotCreated.de_json(json_dict, offline_bot)
        assert managed_bot_created.api_kwargs == {}
        assert managed_bot_created.bot.id == self.bot_id
        assert managed_bot_created.bot.is_bot is True
        assert managed_bot_created.bot.first_name == self.bot_first_name
        assert managed_bot_created.bot.username == self.bot_username

    def test_to_dict(self, managed_bot_created):
        managed_bot_created_dict = managed_bot_created.to_dict()
        assert managed_bot_created_dict["bot"]["id"] == self.bot_id
        assert managed_bot_created_dict["bot"]["is_bot"] is True
        assert managed_bot_created_dict["bot"]["first_name"] == self.bot_first_name
        assert managed_bot_created_dict["bot"]["username"] == self.bot_username

    def test_equality(self, managed_bot_created):
        managed_bot_created_2 = ManagedBotCreated(
            bot=User(
                id=self.bot_id,
                is_bot=True,
                first_name=self.bot_first_name,
                username=self.bot_username,
            ),
        )
        managed_bot_created_3 = ManagedBotCreated(
            bot=User(
                id=4534, is_bot=True, first_name=self.bot_first_name, username=self.bot_username
            ),
        )
        not_a_managed_bot_created = ManagedBotUpdated(
            user=User(
                id=self.bot_creator_id,
                is_bot=True,
                first_name=self.bot_creator_first_name,
                username=self.bot_creator_username,
            ),
            bot=User(
                id=self.bot_id,
                is_bot=True,
                first_name=self.bot_first_name,
                username=self.bot_username,
            ),
        )

        assert managed_bot_created == managed_bot_created_2
        assert hash(managed_bot_created) == hash(managed_bot_created_2)

        assert managed_bot_created != managed_bot_created_3
        assert hash(managed_bot_created) != hash(managed_bot_created_3)

        assert managed_bot_created != not_a_managed_bot_created
        assert hash(managed_bot_created) != hash(not_a_managed_bot_created)


class TestManagedBotUpdatedWithoutRequest(ManagedBotTestBase):
    def test_slot_behaviour(self, managed_bot_updated):
        inst = managed_bot_updated
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot, managed_bot_updated):
        json_dict = {
            "user": managed_bot_updated.user.to_dict(),
            "bot": managed_bot_updated.bot.to_dict(),
        }
        managed_bot_updated_new = ManagedBotUpdated.de_json(json_dict, offline_bot)
        assert managed_bot_updated_new.api_kwargs == {}
        assert managed_bot_updated_new.user == managed_bot_updated.user
        assert managed_bot_updated_new.bot == managed_bot_updated.bot

    def test_to_dict(self, managed_bot_updated):
        managed_bot_updated_dict = managed_bot_updated.to_dict()
        assert managed_bot_updated_dict["user"] == managed_bot_updated.user.to_dict()
        assert managed_bot_updated_dict["bot"] == managed_bot_updated.bot.to_dict()

    def test_equality(self, managed_bot_updated):
        managed_bot_updated_2 = ManagedBotUpdated(
            user=User(
                id=self.bot_creator_id,
                is_bot=True,
                first_name=self.bot_creator_first_name,
                username=self.bot_creator_username,
            ),
            bot=User(
                id=self.bot_id,
                is_bot=True,
                first_name=self.bot_first_name,
                username=self.bot_username,
            ),
        )
        managed_bot_updated_3 = ManagedBotUpdated(
            user=User(
                id=4534,
                is_bot=True,
                first_name=self.bot_creator_first_name,
                username=self.bot_creator_username,
            ),
            bot=User(
                id=self.bot_id,
                is_bot=True,
                first_name=self.bot_first_name,
                username=self.bot_username,
            ),
        )
        not_a_managed_bot_updated = ManagedBotCreated(
            bot=User(
                id=self.bot_id,
                is_bot=True,
                first_name=self.bot_first_name,
                username=self.bot_username,
            ),
        )

        assert managed_bot_updated == managed_bot_updated_2
        assert hash(managed_bot_updated) == hash(managed_bot_updated_2)

        assert managed_bot_updated != managed_bot_updated_3
        assert hash(managed_bot_updated) != hash(managed_bot_updated_3)

        assert managed_bot_updated != not_a_managed_bot_updated
        assert hash(managed_bot_updated) != hash(not_a_managed_bot_updated)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram Poll."""

import datetime as dtm
from collections.abc import Sequence
from typing import TYPE_CHECKING, Final

from telegram import constants
from telegram._chat import Chat
from telegram._files.animation import Animation
from telegram._files.audio import Audio
from telegram._files.document import Document
from telegram._files.livephoto import LivePhoto
from telegram._files.location import Location
from telegram._files.photosize import PhotoSize
from telegram._files.sticker import Sticker
from telegram._files.venue import Venue
from telegram._files.video import Video
from telegram._messageentity import MessageEntity
from telegram._telegramobject import TelegramObject
from telegram._user import User
from telegram._utils import enum
from telegram._utils.argumentparsing import (
    de_json_optional,
    de_list_optional,
    parse_sequence_arg,
    to_timedelta,
)
from telegram._utils.datetime import (
    extract_tzinfo_from_defaults,
    from_timestamp,
    get_timedelta_value,
)
from telegram._utils.defaultvalue import DEFAULT_NONE
from telegram._utils.entities import parse_message_entities, parse_message_entity
from telegram._utils.types import JSONDict, ODVInput, TimePeriod
from telegram._utils.warnings import warn
from telegram.warnings import PTBDeprecationWarning

if TYPE_CHECKING:
    from telegram import Bot, InputPollOptionMedia, MaybeInaccessibleMessage


class PollMedia(TelegramObject):
    """
    At most one of the optional fields can be present in any given object.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if all of their attributes are equal.

    .. versionadded:: 22.8

    Args:
        animation (:class:`telegram.Animation`, optional): Media is an animation, information about
            the animation
        audio (:class:`telegram.Audio`, optional): Media is an audio file, information about the
            file; currently, can't be received in a poll option
        document (:class:`telegram.Document`, optional): Media is a general file, information about
            the file; currently, can't be received in a poll option
        live_photo (:class:`telegram.LivePhoto`, optional): Media is a live photo, information
            about the live photo
        location (:class:`telegram.Location`, optional): Media is a shared location, information
            about the location
        photo (Sequence[:class:`telegram.PhotoSize`], optional): Media is a photo, available sizes
            of the photo
        sticker (:class:`telegram.Sticker`, optional): Media is a sticker, information about the
            sticker; currently, for poll options only
        venue (:class:`telegram.Venue`, optional): Media is a venue, information about the venue
        video (:class:`telegram.Video`, optional): Media is a video, information about the video

    Attributes:
        animation (:class:`telegram.Animation`): Optional. Media is an animation, information about
            the animation
        audio (:class:`telegram.Audio`): Optional. Media is an audio file, information about the
            file; currently, can't be received in a poll option
        document (:class:`telegram.Document`): Optional. Media is a general file, information about
            the file; currently, can't be received in a poll option
        live_photo (:class:`telegram.LivePhoto`, optional): Media is a live photo, information
            about the live photo
        location (:class:`telegram.Location`): Optional. Media is a shared location, information
            about the location
        photo (tuple[:class:`telegram.PhotoSize`]): Optional. Media is a photo, available sizes
            of the photo
        sticker (:class:`telegram.Sticker`): Optional. Media is a sticker, information about the
            sticker; currently, for poll options only
        venue (:class:`telegram.Venue`): Optional. Media is a venue, information about the venue
        video (:class:`telegram.Video`): Optional. Media is a video, information about the video
    """

    __slots__ = (
        "animation",
        "audio",
        "document",
        "live_photo",
        "location",
        "photo",
        "sticker",
        "venue",
        "video",
    )

    def __init__(
        self,
        animation: Animation | None = None,
        audio: Audio | None = None,
        document: Document | None = None,
        live_photo: LivePhoto | None = None,
        location: Location | None = None,
        photo: Sequence[PhotoSize] | None = None,
        sticker: Sticker | None = None,
        venue: Venue | None = None,
        video: Video | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.animation: Animation | None = animation
        self.audio: Audio | None = audio
        self.document: Document | None = document
        self.live_photo: LivePhoto | None = live_photo
        self.location: Location | None = location
        self.photo: tuple[PhotoSize, ...] = parse_sequence_arg(photo)
        self.sticker: Sticker | None = sticker
        self.venue: Venue | None = venue
        self.video: Video | None = video

        self._id_attrs = (
            self.animation,
            self.audio,
            self.document,
            self.live_photo,
            self.location,
            self.photo,
            self.sticker,
            self.venue,
            self.video,
        )

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "PollMedia":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["animation"] = de_json_optional(data.get("animation"), Animation, bot)
        data["audio"] = de_json_optional(data.get("audio"), Audio, bot)
        data["document"] = de_json_optional(data.get("document"), Document, bot)
        data["live_photo"] = de_json_optional(data.get("live_photo"), LivePhoto, bot)
        data["location"] = de_json_optional(data.get("location"), Location, bot)
        data["photo"] = de_list_optional(data.get("photo"), PhotoSize, bot)
        data["sticker"] = de_json_optional(data.get("sticker"), Sticker, bot)
        data["venue"] = de_json_optional(data.get("venue"), Venue, bot)
        data["video"] = de_json_optional(data.get("video"), Video, bot)

        return super().de_json(data=data, bot=bot)


class InputPollOption(TelegramObject):
    """
    This object contains information about one answer option in a poll to be sent.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`text` is equal.

    .. versionadded:: 21.2

    Args:
        text (:obj:`str`): Option text,
            :tg-const:`telegram.PollOption.MIN_LENGTH`-:tg-const:`telegram.PollOption.MAX_LENGTH`
            characters.
        text_parse_mode (:obj:`str`, optional): |parse_mode|
            Currently, only custom emoji entities are allowed.
        text_entities (Sequence[:class:`telegram.MessageEntity`], optional): Special entities
            that appear in the option :paramref:`text`. It can be specified instead of
            :paramref:`text_parse_mode`.
            Currently, only custom emoji entities are allowed.
            This list is empty if the text does not contain entities.
        media (:class:`telegram.InputPollOptionMedia`, optional): Media added to the poll option.

            .. versionadded:: 22.8

    Attributes:
        text (:obj:`str`): Option text,
            :tg-const:`telegram.PollOption.MIN_LENGTH`-:tg-const:`telegram.PollOption.MAX_LENGTH`
            characters.
        text_parse_mode (:obj:`str`): Optional. |parse_mode|
            Currently, only custom emoji entities are allowed.
        text_entities (Sequence[:class:`telegram.MessageEntity`]): Special entities
            that appear in the option :paramref:`text`. It can be specified instead of
            :paramref:`text_parse_mode`.
            Currently, only custom emoji entities are allowed.
            This list is empty if the text does not contain entities.
        media (:class:`telegram.InputPollOptionMedia`): Optional. Media added to the poll option.

            .. versionadded:: 22.8
    """

    __slots__ = ("media", "text", "text_entities", "text_parse_mode")

    def __init__(
        self,
        text: str,
        text_parse_mode: ODVInput[str] = DEFAULT_NONE,
        text_entities: Sequence[MessageEntity] | None = None,
        media: "InputPollOptionMedia | None" = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.text: str = text
        self.text_parse_mode: ODVInput[str] = text_parse_mode
        self.text_entities: tuple[MessageEntity, ...] = parse_sequence_arg(text_entities)
        self.media: InputPollOptionMedia | None = media

        self._id_attrs = (self.text,)

        self._freeze()

    # tags: deprecated 22.8
    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "InputPollOption":
        """See :meth:`telegram.TelegramObject.de_json`. The :paramref:`media` field will
        not be included for deserialization.

        .. deprecated:: 22.8
            This class is input only and will be removed in the next version.
        """
        warn(
            PTBDeprecationWarning(
                "22.8",
                "`InputPollOption.de_json` is deprecated. This class is input only and will be "
                "removed in the next version. The `media` field will not be included for "
                "deserialization.",
            ),
            stacklevel=2,
        )
        data = cls._parse_data(data)

        data["text_entities"] = de_list_optional(data.get("text_entities"), MessageEntity, bot)

        return super().de_json(data=data, bot=bot)


class PollOption(TelegramObject):
    """
    This object contains information about one answer option in a poll.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`text`, :attr:`voter_count` and :attr:`persistent_id`
    are equal.

    .. versionchanged:: 22.8
        Added attribute :attr:`persistent_id` to equality checks.


    Args:
        persistent_id (:obj:`str`): Unique identifier of the option, persistent on option addition
            and deletion.

            .. versionadded:: 22.8
        text (:obj:`str`): Option text,
            :tg-const:`telegram.PollOption.MIN_LENGTH`-:tg-const:`telegram.PollOption.MAX_LENGTH`
            characters.
        voter_count (:obj:`int`): Number of users that voted for this option.
        text_entities (Sequence[:class:`telegram.MessageEntity`], optional): Special entities
            that appear in the option text. Currently, only custom emoji entities are allowed in
            poll option texts.

            .. versionadded:: 21.2
        media (:class:`telegram.PollMedia`, optional): Media added to the poll option.

            .. versionadded:: 22.8
        added_by_user (:class:`telegram.User`, optional): User who added the option;
            omitted if the option wasn't added by a user after poll creation.

            .. versionadded:: 22.8
        added_by_chat (:class:`telegram.Chat`, optional): Chat that added the option;
            omitted if the option wasn't added by a chat after poll creation.

            .. versionadded:: 22.8
        addition_date (:obj:`datetime.datetime`, optional): Point in time
            when the option was added; omitted if the option existed in the original poll.

            .. versionadded:: 22.8

    Attributes:
        persistent_id (:obj:`str`): Unique identifier of the option, persistent on option addition
            and deletion.

            .. versionadded:: 22.8
        text (:obj:`str`): Option text,
            :tg-const:`telegram.PollOption.MIN_LENGTH`-:tg-const:`telegram.PollOption.MAX_LENGTH`
            characters.
        voter_count (:obj:`int`): Number of users that voted for this option.
        text_entities (tuple[:class:`telegram.MessageEntity`]): Special entities
            that appear in the option text. Currently, only custom emoji entities are allowed in
            poll option texts.
            This list is empty if the question does not contain entities.

            .. versionadded:: 21.2
        media (:class:`telegram.PollMedia`): Optional. Media added to the poll option.

            .. versionadded:: 22.8
        added_by_user (:class:`telegram.User`): Optional. User who added the option;
            omitted if the option wasn't added by a user after poll creation.

            .. versionadded:: 22.8
        added_by_chat (:class:`telegram.Chat`): Optional. Chat that added the option;
            omitted if the option wasn't added by a chat after poll creation.

            .. versionadded:: 22.8
        addition_date (:obj:`datetime.datetime`): Optional. Point in time
            when the option was added; omitted if the option existed in the original poll.

            .. versionadded:: 22.8
    """

    __slots__ = (
        "added_by_chat",
        "added_by_user",
        "addition_date",
        "media",
        "persistent_id",
        "text",
        "text_entities",
        "voter_count",
    )

    def __init__(
        self,
        text: str,
        voter_count: int,
        text_entities: Sequence[MessageEntity] | None = None,
        added_by_user: User | None = None,
        added_by_chat: Chat | None = None,
        addition_date: dtm.datetime | None = None,
        media: PollMedia | None = None,
        # tags: required in 22.8, bot api 9.6
        # temporarily optional to avoid breaking changes
        persistent_id: str | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        if persistent_id is None:
            raise TypeError("`persistent_id` is a required argument since Bot API 9.6")

        super().__init__(api_kwargs=api_kwargs)
        self.text: str = text
        self.voter_count: int = voter_count
        self.added_by_user: User | None = added_by_user
        self.added_by_chat: Chat | None = added_by_chat
        self.addition_date: dtm.datetime | None = addition_date
        self.persistent_id: str = persistent_id
        self.media: PollMedia | None = media

        self.text_entities: tuple[MessageEntity, ...] = parse_sequence_arg(text_entities)

        self._id_attrs = (self.text, self.voter_count, self.persistent_id)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "PollOption":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # Get the local timezone from the bot if it has defaults
        loc_tzinfo = extract_tzinfo_from_defaults(bot)

        data["text_entities"] = de_list_optional(data.get("text_entities"), MessageEntity, bot)
        data["added_by_user"] = de_json_optional(data.get("added_by_user"), User, bot)
        data["added_by_chat"] = de_json_optional(data.get("added_by_chat"), Chat, bot)
        data["addition_date"] = from_timestamp(data.get("addition_date"), tzinfo=loc_tzinfo)
        data["media"] = de_json_optional(data.get("media"), PollMedia, bot)

        return super().de_json(data=data, bot=bot)

    def parse_entity(self, entity: MessageEntity) -> str:
        """Returns the text in :attr:`text`
        from a given :class:`telegram.MessageEntity` of :attr:`text_entities`.

        Note:
            This method is present because Telegram calculates the offset and length in
            UTF-16 codepoint pairs, which some versions of Python don't handle automatically.
            (That is, you can't just slice ``Message.text`` with the offset and length.)

        .. versionadded:: 21.2

        Args:
            entity (:class:`telegram.MessageEntity`): The entity to extract the text from. It must
                be an entity that belongs to :attr:`text_entities`.

        Returns:
            :obj:`str`: The text of the given entity.
        """
        return parse_message_entity(self.text, entity)

    def parse_entities(self, types: list[str] | None = None) -> dict[MessageEntity, str]:
        """
        Returns a :obj:`dict` that maps :class:`telegram.MessageEntity` to :obj:`str`.
        It contains entities from this polls question filtered by their ``type`` attribute as
        the key, and the text that each entity belongs to as the value of the :obj:`dict`.

        Note:
            This method should always be used instead of the :attr:`text_entities`
            attribute, since it calculates the correct substring from the message text based on
            UTF-16 codepoints. See :attr:`parse_entity` for more info.

        .. versionadded:: 21.2

        Args:
            types (list[:obj:`str`], optional): List of ``MessageEntity`` types as strings. If the
                    ``type`` attribute of an entity is contained in this list, it will be returned.
                    Defaults to :attr:`telegram.MessageEntity.ALL_TYPES`.

        Returns:
            dict[:class:`telegram.MessageEntity`, :obj:`str`]: A dictionary of entities mapped to
            the text that belongs to them, calculated based on UTF-16 codepoints.
        """
        return parse_message_entities(self.text, self.text_entities, types)

    MIN_LENGTH: Final[int] = constants.PollLimit.MIN_OPTION_LENGTH
    """:const:`telegram.constants.PollLimit.MIN_OPTION_LENGTH`

    .. versionadded:: 20.0
    """
    MAX_LENGTH: Final[int] = constants.PollLimit.MAX_OPTION_LENGTH
    """:const:`telegram.constants.PollLimit.MAX_OPTION_LENGTH`

    .. versionadded:: 20.0
    """


class PollAnswer(TelegramObject):
    """
    This object represents an answer of a user in a non-anonymous poll.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`poll_id`, :attr:`user` and :attr:`option_ids` are equal.

    .. versionchanged:: 20.5
        The order of :paramref:`option_ids` and :paramref:`user` is changed in
        20.5 as the latter one became optional.

    .. versionchanged:: 20.6
       Backward compatiblity for changed order of :paramref:`option_ids` and :paramref:`user`
       was removed.

    Args:
        poll_id (:obj:`str`): Unique poll identifier.
        option_ids (Sequence[:obj:`int`]): Identifiers of answer options, chosen by the user. May
            be empty if the user retracted their vote.

            .. versionchanged:: 20.0
                |sequenceclassargs|
        option_persistent_ids (Sequence[:obj:`str`]): Persistent identifiers of the
            chosen answer options. May be empty if the vote was retracted.

            .. versionadded:: 22.8
        user (:class:`telegram.User`, optional): The user that changed the answer to the poll,
            if the voter isn't anonymous. If the voter is anonymous, this field will contain the
            user :tg-const:`telegram.constants.ChatID.FAKE_CHANNEL` for backwards compatibility.

            .. versionchanged:: 20.5
                :paramref:`user` became optional.
        voter_chat (:class:`telegram.Chat`, optional): The chat that changed the answer to the
            poll, if the voter is anonymous.

            .. versionadded:: 20.5

    Attributes:
        poll_id (:obj:`str`): Unique poll identifier.
        option_ids (tuple[:obj:`int`]): Identifiers of answer options, chosen by the user. May
            be empty if the user retracted their vote.

            .. versionchanged:: 20.0
                |tupleclassattrs|
        option_persistent_ids (tuple[:obj:`str`]): Persistent identifiers of the
            chosen answer options. May be empty if the vote was retracted.

            .. versionadded:: 22.8
        user (:class:`telegram.User`): Optional. The user, who changed the answer to the
            poll, if the voter isn't anonymous. If the voter is anonymous, this field will contain
            the user :tg-const:`telegram.constants.ChatID.FAKE_CHANNEL` for backwards compatibility

            .. versionchanged:: 20.5
                :paramref:`user` became optional.
        voter_chat (:class:`telegram.Chat`): Optional. The chat that changed the answer to the
            poll, if the voter is anonymous.

            .. versionadded:: 20.5
    """

    __slots__ = ("option_ids", "option_persistent_ids", "poll_id", "user", "voter_chat")

    def __init__(
        self,
        poll_id: str,
        option_ids: Sequence[int],
        user: User | None = None,
        voter_chat: Chat | None = None,
        # tags: required in 22.8, bot api 9.6
        # temporarily optional to avoid breaking changes
        option_persistent_ids: Sequence[str] | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        if option_persistent_ids is None:
            raise TypeError("`option_persistent_ids` is a required argument since Bot API 9.6")

        super().__init__(api_kwargs=api_kwargs)
        self.poll_id: str = poll_id
        self.voter_chat: Chat | None = voter_chat
        self.option_ids: tuple[int, ...] = parse_sequence_arg(option_ids)
        self.user: User | None = user
        self.option_persistent_ids: tuple[str, ...] = parse_sequence_arg(option_persistent_ids)

        self._id_attrs = (
            self.poll_id,
            self.option_ids,
            self.user,
            self.voter_chat,
        )

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "PollAnswer":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["user"] = de_json_optional(data.get("user"), User, bot)
        data["voter_chat"] = de_json_optional(data.get("voter_chat"), Chat, bot)

        return super().de_json(data=data, bot=bot)


class PollOptionAdded(TelegramObject):
    """
    Describes a service message about an option added to a poll.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`option_persistent_id`, and :attr:`option_text` are equal.

    .. versionadded:: 22.8

    Args:
        option_persistent_id (:obj:`str`): Unique identifier of the added option.
        option_text (:obj:`str`): Option text.
        poll_message (:class:`telegram.MaybeInaccessibleMessage`, optional): Message
            containing the poll to which the option was added, if known.
            Note that the Message object in this field will not contain the
            :attr:`~telegram.Message.reply_to_message` field even if it itself is a reply.
        option_text_entities (Sequence[:class:`telegram.MessageEntity`], optional): Special
            entities that appear in the :paramref:`option_text`.

    Attributes:
        option_persistent_id (:obj:`str`): Unique identifier of the added option.
        option_text (:obj:`str`): Option text.
        poll_message (:class:`telegram.MaybeInaccessibleMessage`): Optional. Message
            containing the poll to which the option was added, if known.
            Note that the Message object in this field will not contain the
            :attr:`~telegram.Message.reply_to_message` field even if it itself is a reply.
        option_text_entities (tuple[:class:`telegram.MessageEntity`]): Optional. Special
            entities that appear in the :paramref:`option_text`.
    """

    __slots__ = ("option_persistent_id", "option_text", "option_text_entities", "poll_message")

    def __init__(
        self,
        option_persistent_id: str,
        option_text: str,
        poll_message: "MaybeInaccessibleMessage | None" = None,
        option_text_entities: Sequence[MessageEntity] | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.option_persistent_id: str = option_persistent_id
        self.option_text: str = option_text
        self.poll_message: MaybeInaccessibleMessage | None = poll_message

        self.option_text_entities: tuple[MessageEntity, ...] = parse_sequence_arg(
            option_text_entities
        )

        self._id_attrs = (self.option_persistent_id, self.option_text)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "PollOptionAdded":
        """See :meth:`telegram.TelegramObject.de_json`."""
        from telegram._message import (  # pylint: disable=import-outside-toplevel  # noqa: PLC0415
            MaybeInaccessibleMessage,
        )

        data = cls._parse_data(data)

        data["poll_message"] = de_json_optional(
            data.get("poll_message"), MaybeInaccessibleMessage, bot
        )
        data["option_text_entities"] = de_list_optional(
            data.get("option_text_entities"), MessageEntity, bot
        )

        return super().de_json(data=data, bot=bot)

    def parse_option_text_entity(self, entity: MessageEntity) -> str:
        """Returns the text in :attr:`option_text`
        from a given :class:`telegram.MessageEntity` of :attr:`option_text_entities`.

        Note:
            This method is present because Telegram calculates the offset and length in
            UTF-16 codepoint pairs, which some versions of Python don't handle automatically.
            (That is, you can't just slice ``Message.text`` with the offset and length.)

        Args:
            entity (:class:`telegram.MessageEntity`): The entity to extract the text from. It must
                be an entity that belongs to :attr:`option_text_entities`.

        Returns:
            :obj:`str`: The text of the given entity.
        """
        return parse_message_entity(self.option_text, entity)

    def parse_option_text_entities(
        self, types: list[str] | None = None
    ) -> dict[MessageEntity, str]:
        """
        Returns a :obj:`dict` that maps :class:`telegram.MessageEntity` to :obj:`str`.
        It contains entities from this polls option text filtered by their ``type`` attribute as
        the key, and the text that each entity belongs to as the value of the :obj:`dict`.

        Note:
            This method should always be used instead of the :attr:`option_text_entities`
            attribute, since it calculates the correct substring from the message text based on
            UTF-16 codepoints. See :attr:`parse_entity` for more info.

        Args:
            types (list[:obj:`str`], optional): List of ``MessageEntity`` types as strings. If the
                    ``type`` attribute of an entity is contained in this list, it will be returned.
                    Defaults to :attr:`telegram.MessageEntity.ALL_TYPES`.

        Returns:
            dict[:class:`telegram.MessageEntity`, :obj:`str`]: A dictionary of entities mapped to
            the text that belongs to them, calculated based on UTF-16 codepoints.
        """
        return parse_message_entities(self.option_text, self.option_text_entities, types)


class PollOptionDeleted(TelegramObject):
    """
    Describes a service message about an option deleted from a poll.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`option_persistent_id`, :attr:`option_text` are equal.

    .. versionadded:: 22.8

    Args:
        option_persistent_id (:obj:`str`): Unique identifier of the deleted option.
        option_text (:obj:`str`): Option text.
        poll_message (:class:`telegram.MaybeInaccessibleMessage`, optional): Message
            containing the poll to which the option was deleted, if known.
            Note that the Message object in this field will not contain the
            :attr:`~telegram.Message.reply_to_message` field even if it itself is a reply.
        option_text_entities (Sequence[:class:`telegram.MessageEntity`], optional): Special
            entities that appear in the option_text.

    Attributes:
        option_persistent_id (:obj:`str`): Unique identifier of the deleted option.
        option_text (:obj:`str`): Option text.
        poll_message (:class:`telegram.MaybeInaccessibleMessage`): Optional. Message
            containing the poll to which the option was deleted, if known.
            Note that the Message object in this field will not contain the
            :attr:`~telegram.Message.reply_to_message` field even if it itself is a reply.
        option_text_entities (tuple[:class:`telegram.MessageEntity`]): Optional. Special
            entities that appear in the option_text.
    """

    __slots__ = ("option_persistent_id", "option_text", "option_text_entities", "poll_message")

    def __init__(
        self,
        option_persistent_id: str,
        option_text: str,
        poll_message: "MaybeInaccessibleMessage | None" = None,
        option_text_entities: Sequence[MessageEntity] | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.option_persistent_id: str = option_persistent_id
        self.option_text: str = option_text
        self.poll_message: MaybeInaccessibleMessage | None = poll_message

        self.option_text_entities: tuple[MessageEntity, ...] = parse_sequence_arg(
            option_text_entities
        )

        self._id_attrs = (self.option_persistent_id, self.option_text)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "PollOptionDeleted":
        """See :meth:`telegram.TelegramObject.de_json`."""
        from telegram._message import (  # pylint: disable=import-outside-toplevel  # noqa: PLC0415
            MaybeInaccessibleMessage,
        )

        data = cls._parse_data(data)

        data["poll_message"] = de_json_optional(
            data.get("poll_message"), MaybeInaccessibleMessage, bot
        )
        data["option_text_entities"] = de_list_optional(
            data.get("option_text_entities"), MessageEntity, bot
        )

        return super().de_json(data=data, bot=bot)

    def parse_option_text_entity(self, entity: MessageEntity) -> str:
        """Returns the text in :attr:`option_text`
        from a given :class:`telegram.MessageEntity` of :attr:`option_text_entities`.

        Note:
            This method is present because Telegram calculates the offset and length in
            UTF-16 codepoint pairs, which some versions of Python don't handle automatically.
            (That is, you can't just slice ``Message.text`` with the offset and length.)

        Args:
            entity (:class:`telegram.MessageEntity`): The entity to extract the text from. It must
                be an entity that belongs to :attr:`option_text_entities`.

        Returns:
            :obj:`str`: The text of the given entity.
        """
        return parse_message_entity(self.option_text, entity)

    def parse_option_text_entities(
        self, types: list[str] | None = None
    ) -> dict[MessageEntity, str]:
        """
        Returns a :obj:`dict` that maps :class:`telegram.MessageEntity` to :obj:`str`.
        It contains entities from this polls option text filtered by their ``type`` attribute as
        the key, and the text that each entity belongs to as the value of the :obj:`dict`.

        Note:
            This method should always be used instead of the :attr:`option_text_entities`
            attribute, since it calculates the correct substring from the message text based on
            UTF-16 codepoints. See :attr:`parse_entity` for more info.

        Args:
            types (list[:obj:`str`], optional): List of ``MessageEntity`` types as strings. If the
                    ``type`` attribute of an entity is contained in this list, it will be returned.
                    Defaults to :attr:`telegram.MessageEntity.ALL_TYPES`.

        Returns:
            dict[:class:`telegram.MessageEntity`, :obj:`str`]: A dictionary of entities mapped to
            the text that belongs to them, calculated based on UTF-16 codepoints.
        """
        return parse_message_entities(self.option_text, self.option_text_entities, types)


class Poll(TelegramObject):
    """
    This object contains information about a poll.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`id` is equal.

    Examples:
        :any:`Poll Bot <examples.pollbot>`

    Args:
        id (:obj:`str`): Unique poll identifier.
        question (:obj:`str`): Poll question, :tg-const:`telegram.Poll.MIN_QUESTION_LENGTH`-
            :tg-const:`telegram.Poll.MAX_QUESTION_LENGTH` characters.
        options (Sequence[:class:`~telegram.PollOption`]): List of poll options.

            .. versionchanged:: 20.0
                |sequenceclassargs|
        is_closed (:obj:`bool`): :obj:`True`, if the poll is closed.
        is_anonymous (:obj:`bool`): :obj:`True`, if the poll is anonymous.
        type (:obj:`str`): Poll type, currently can be :attr:`REGULAR` or :attr:`QUIZ`.
        allows_multiple_answers (:obj:`bool`): :obj:`True`, if the poll allows multiple answers.
        members_only (:obj:`bool`): :obj:`True`, if voting is limited to users who have been
            members of the chat where the poll was originally sent for more than
            :tg-const:`telegram.Poll.MIN_MEMBERSHIP_HOURS` hours.

            .. versionadded:: 22.8
        correct_option_id (:obj:`int`, optional): A zero based identifier of the correct answer
            option. Available only for closed polls in the quiz mode, which were sent
            (not forwarded), by the bot or to a private chat with the bot.

            .. deprecated:: 22.8
                Use :paramref:`correct_option_ids` instead.
        explanation (:obj:`str`, optional): Text that is shown when a user chooses an incorrect
            answer or taps on the lamp icon in a quiz-style poll,
            0-:tg-const:`telegram.Poll.MAX_EXPLANATION_LENGTH` characters.
        explanation_entities (Sequence[:class:`telegram.MessageEntity`], optional): Special
            entities like usernames, URLs, bot commands, etc. that appear in the
            :attr:`explanation`. This list is empty if the message does not contain explanation
            entities.

            .. versionchanged:: 20.0

               * This attribute is now always a (possibly empty) list and never :obj:`None`.
               * |sequenceclassargs|
        explanation_media (:class:`telegram.PollMedia`, optional): Media added to the quiz
            explanation.

            .. versionadded:: 22.8
        open_period (:obj:`int` | :class:`datetime.timedelta`, optional): Amount of time in seconds
            the poll will be active after creation.

            .. versionchanged:: v22.2
                |time-period-input|
        close_date (:obj:`datetime.datetime`, optional): Point in time (Unix timestamp) when the
            poll will be automatically closed. Converted to :obj:`datetime.datetime`.

            .. versionchanged:: 20.3
                |datetime_localization|
        question_entities (Sequence[:class:`telegram.MessageEntity`], optional): Special entities
            that appear in the :attr:`question`. Currently, only custom emoji entities are allowed
            in poll questions.

            .. versionadded:: 21.2
        allows_revoting (:obj:`bool`, optional): :obj:`True`, if the poll allows to
            change the chosenanswer options.

            .. versionadded:: 22.8
        correct_option_ids (Sequence[:class:`int`], optional): Array of 0-based identifiers of
            the correct answer options. Available only for polls in quiz mode which are closed or
            were sent (not forwarded) by the bot or to the private chat with the bot.

            .. versionadded:: 22.8
        country_codes (Sequence[:obj:`str`], optional): A list of two-letter ``ISO 3166-1 alpha-2``
            country codes indicating the countries from which users can vote in the poll. The
            country code ``"FT"`` is used for users with anonymous numbers. If omitted, then users
            from any country can participate in the poll.

            .. versionadded:: 22.8
        description (:obj:`str`, optional): Description of the poll;
            for polls inside the :class:`~telegram.Message` object only.

            .. versionadded:: 22.8
        description_entities (Sequence[:class:`telegram.MessageEntity`], optional): Special
            entities like usernames, URLs, bot commands, etc. that appear in the description

            .. versionadded:: 22.8
        media (:class:`telegram.PollMedia`, optional): Media added to the poll description;
            for polls inside the :class:`~telegram.Message` object only.

            .. versionadded:: 22.8

    Attributes:
        id (:obj:`str`): Unique poll identifier.
        question (:obj:`str`): Poll question, :tg-const:`telegram.Poll.MIN_QUESTION_LENGTH`-
            :tg-const:`telegram.Poll.MAX_QUESTION_LENGTH` characters.
        options (tuple[:class:`~telegram.PollOption`]): List of poll options.

            .. versionchanged:: 20.0
                |tupleclassattrs|
        total_voter_count (:obj:`int`): Total number of users that voted in the poll.
        is_closed (:obj:`bool`): :obj:`True`, if the poll is closed.
        is_anonymous (:obj:`bool`): :obj:`True`, if the poll is anonymous.
        type (:obj:`str`): Poll type, currently can be :attr:`REGULAR` or :attr:`QUIZ`.
        allows_multiple_answers (:obj:`bool`): :obj:`True`, if the poll allows multiple answers.
        members_only (:obj:`bool`): :obj:`True`, if voting is limited to users who have been
            members of the chat where the poll was originally sent for more than
            :tg-const:`telegram.Poll.MIN_MEMBERSHIP_HOURS` hours.

            .. versionadded:: 22.8
        explanation (:obj:`str`): Optional. Text that is shown when a user chooses an incorrect
            answer or taps on the lamp icon in a quiz-style poll,
            0-:tg-const:`telegram.Poll.MAX_EXPLANATION_LENGTH` characters.
        explanation_entities (tuple[:class:`telegram.MessageEntity`]): Special entities
            like usernames, URLs, bot commands, etc. that appear in the :attr:`explanation`.
            This list is empty if the message does not contain explanation entities.

            .. versionchanged:: 20.0
                |tupleclassattrs|

            .. versionchanged:: 20.0
               This attribute is now always a (possibly empty) list and never :obj:`None`.
        explanation_media (:class:`telegram.PollMedia`): Optional. Media added to the quiz
            explanation.

            .. versionadded:: 22.8
        open_period (:obj:`int` | :class:`datetime.timedelta`): Optional. Amount of time in seconds
            the poll will be active after creation.

            .. deprecated:: v22.2
                |time-period-int-deprecated|
        close_date (:obj:`datetime.datetime`): Optional. Point in time when the poll will be
            automatically closed.

            .. versionchanged:: 20.3
                |datetime_localization|
        question_entities (tuple[:class:`telegram.MessageEntity`]): Special entities
            that appear in the :attr:`question`. Currently, only custom emoji entities are allowed
            in poll questions.
            This list is empty if the question does not contain entities.

            .. versionadded:: 21.2
        allows_revoting (:obj:`bool`): :obj:`True`, if the poll
            allows to change the chosenanswer options

            .. versionadded:: 22.8
        correct_option_ids (tuple[:class:`int`]): Array of 0-based identifiers of the
            correct answer options. Available only for polls in quiz mode which are closed or were
            sent (not forwarded) by the bot or to the private chat with the bot.

            .. versionadded:: 22.8
        country_codes (tuple[:obj:`str`]): Optional. A list of two-letter ``ISO 3166-1 alpha-2``
            country codes indicating the countries from which users can vote in the poll. The
            country code ``"FT"`` is used for users with anonymous numbers. If omitted, then users
            from any country can participate in the poll.

            .. versionadded:: 22.8
        description (:obj:`str`): Optional. Description of the poll;
            for polls inside the Message object only

            .. versionadded:: 22.8
        description_entities (tuple[:class:`telegram.MessageEntity`]): Special
            entities like usernames, URLs, bot commands, etc. that appear in the description

            .. versionadded:: 22.8
        media (:class:`telegram.PollMedia`): Optional. Media added to the poll description;
            for polls inside the Message object only.

            .. versionadded:: 22.8

    """

    __slots__ = (
        "_open_period",
        "allows_multiple_answers",
        "allows_revoting",
        "close_date",
        "correct_option_ids",
        "country_codes",
        "description",
        "description_entities",
        "explanation",
        "explanation_entities",
        "explanation_media",
        "id",
        "is_anonymous",
        "is_closed",
        "media",
        "members_only",
        "options",
        "question",
        "question_entities",
        "total_voter_count",
        "type",
    )

    def __init__(
        self,
        id: str,  # pylint: disable=redefined-builtin
        question: str,
        options: Sequence[PollOption],
        total_voter_count: int,
        is_closed: bool,
        is_anonymous: bool,
        type: str,  # pylint: disable=redefined-builtin
        allows_multiple_answers: bool,
        # tags: deprecated 22.8
        # Removed in bot api 9.6:
        correct_option_id: int | None = None,
        # ---
        explanation: str | None = None,
        explanation_entities: Sequence[MessageEntity] | None = None,
        open_period: TimePeriod | None = None,
        close_date: dtm.datetime | None = None,
        question_entities: Sequence[MessageEntity] | None = None,
        # tags: required in 22.8
        # temporarily optional to avoid breaking changes
        allows_revoting: bool | None = None,
        members_only: bool | None = None,
        # ---
        correct_option_ids: Sequence[int] | None = None,
        description: str | None = None,
        description_entities: Sequence[MessageEntity] | None = None,
        country_codes: Sequence[str] | None = None,
        media: PollMedia | None = None,
        explanation_media: PollMedia | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        if allows_revoting is None:
            raise TypeError("`allows_revoting` is a required argument since Bot API 9.6")

        if members_only is None:
            raise TypeError("`members_only` is a required argument since Bot API 10.0")

        super().__init__(api_kwargs=api_kwargs)
        self.id: str = id
        self.question: str = question
        self.options: tuple[PollOption, ...] = parse_sequence_arg(options)
        self.total_voter_count: int = total_voter_count
        self.is_closed: bool = is_closed
        self.is_anonymous: bool = is_anonymous
        self.type: str = enum.get_member(constants.PollType, type, type)
        self.allows_multiple_answers: bool = allows_multiple_answers
        self.allows_revoting: bool = allows_revoting
        self.members_only: bool = members_only

        # tag: deprecated 22.8
        if correct_option_id is not None:
            warn(
                PTBDeprecationWarning(
                    "22.8",
                    "The parameter `correct_option_id` is deprecated. "
                    "Use `correct_option_ids` instead.",
                ),
                stacklevel=2,
            )
            if correct_option_ids is None:
                correct_option_ids = [correct_option_id]

        self.correct_option_ids: tuple[int, ...] = parse_sequence_arg(correct_option_ids)
        self.description: str | None = description
        self.description_entities: tuple[MessageEntity, ...] = parse_sequence_arg(
            description_entities
        )
        self.explanation: str | None = explanation
        self.explanation_entities: tuple[MessageEntity, ...] = parse_sequence_arg(
            explanation_entities
        )
        self._open_period: dtm.timedelta | None = to_timedelta(open_period)
        self.close_date: dtm.datetime | None = close_date
        self.question_entities: tuple[MessageEntity, ...] = parse_sequence_arg(question_entities)
        self.country_codes: tuple[str, ...] = parse_sequence_arg(country_codes)
        self.media: PollMedia | None = media
        self.explanation_media: PollMedia | None = explanation_media

        self._id_attrs = (self.id,)

        self._freeze()

    @property
    def open_period(self) -> int | dtm.timedelta | None:
        return get_timedelta_value(self._open_period, attribute="open_period")

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "Poll":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        # Get the local timezone from the bot if it has defaults
        loc_tzinfo = extract_tzinfo_from_defaults(bot)

        data["options"] = de_list_optional(data.get("options"), PollOption, bot)
        data["explanation_entities"] = de_list_optional(
            data.get("explanation_entities"), MessageEntity, bot
        )
        data["close_date"] = from_timestamp(data.get("close_date"), tzinfo=loc_tzinfo)
        data["question_entities"] = de_list_optional(
            data.get("question_entities"), MessageEntity, bot
        )
        data["description_entities"] = de_list_optional(
            data.get("description_entities"), MessageEntity, bot
        )
        data["media"] = de_json_optional(data.get("media"), PollMedia, bot)
        data["explanation_media"] = de_json_optional(data.get("explanation_media"), PollMedia, bot)

        return super().de_json(data=data, bot=bot)

    def parse_explanation_entity(self, entity: MessageEntity) -> str:
        """Returns the text in :attr:`explanation` from a given :class:`telegram.MessageEntity` of
        :attr:`explanation_entities`.

        Note:
            This method is present because Telegram calculates the offset and length in
            UTF-16 codepoint pairs, which some versions of Python don't handle automatically.
            (That is, you can't just slice ``Message.text`` with the offset and length.)

        Args:
            entity (:class:`telegram.MessageEntity`): The entity to extract the text from. It must
                be an entity that belongs to :attr:`explanation_entities`.

        Returns:
            :obj:`str`: The text of the given entity.

        Raises:
            RuntimeError: If the poll has no explanation.

        """
        if not self.explanation:
            raise RuntimeError("This Poll has no 'explanation'.")

        return parse_message_entity(self.explanation, entity)

    def parse_explanation_entities(
        self, types: list[str] | None = None
    ) -> dict[MessageEntity, str]:
        """
        Returns a :obj:`dict` that maps :class:`telegram.MessageEntity` to :obj:`str`.
        It contains entities from this polls explanation filtered by their ``type`` attribute as
        the key, and the text that each entity belongs to as the value of the :obj:`dict`.

        Note:
            This method should always be used instead of the :attr:`explanation_entities`
            attribute, since it calculates the correct substring from the message text based on
            UTF-16 codepoints. See :attr:`parse_explanation_entity` for more info.

        Args:
            types (list[:obj:`str`], optional): List of ``MessageEntity`` types as strings. If the
                    ``type`` attribute of an entity is contained in this list, it will be returned.
                    Defaults to :attr:`telegram.MessageEntity.ALL_TYPES`.

        Returns:
            dict[:class:`telegram.MessageEntity`, :obj:`str`]: A dictionary of entities mapped to
            the text that belongs to them, calculated based on UTF-16 codepoints.

        Raises:
            RuntimeError: If the poll has no explanation.

        """
        if not self.explanation:
            raise RuntimeError("This Poll has no 'explanation'.")

        return parse_message_entities(self.explanation, self.explanation_entities, types)

    def parse_question_entity(self, entity: MessageEntity) -> str:
        """Returns the text in :attr:`question` from a given :class:`telegram.MessageEntity` of
        :attr:`question_entities`.

        .. versionadded:: 21.2

        Note:
            This method is present because Telegram calculates the offset and length in
            UTF-16 codepoint pairs, which some versions of Python don't handle automatically.
            (That is, you can't just slice ``Message.text`` with the offset and length.)

        Args:
            entity (:class:`telegram.MessageEntity`): The entity to extract the text from. It must
                be an entity that belongs to :attr:`question_entities`.

        Returns:
            :obj:`str`: The text of the given entity.
        """
        return parse_message_entity(self.question, entity)

    def parse_question_entities(self, types: list[str] | None = None) -> dict[MessageEntity, str]:
        """
        Returns a :obj:`dict` that maps :class:`telegram.MessageEntity` to :obj:`str`.
        It contains entities from this polls question filtered by their ``type`` attribute as
        the key, and the text that each entity belongs to as the value of the :obj:`dict`.

        .. versionadded:: 21.2

        Note:
            This method should always be used instead of the :attr:`question_entities`
            attribute, since it calculates the correct substring from the message text based on
            UTF-16 codepoints. See :attr:`parse_question_entity` for more info.

        Args:
            types (list[:obj:`str`], optional): List of ``MessageEntity`` types as strings. If the
                    ``type`` attribute of an entity is contained in this list, it will be returned.
                    Defaults to :attr:`telegram.MessageEntity.ALL_TYPES`.

        Returns:
            dict[:class:`telegram.MessageEntity`, :obj:`str`]: A dictionary of entities mapped to
            the text that belongs to them, calculated based on UTF-16 codepoints.

        """
        return parse_message_entities(self.question, self.question_entities, types)

    def parse_description_entity(self, entity: MessageEntity) -> str:
        """Returns the text in :attr:`description` from a given :class:`telegram.MessageEntity` of
        :attr:`description_entities`.

        .. versionadded:: 22.8

        Note:
            This method is present because Telegram calculates the offset and length in
            UTF-16 codepoint pairs, which some versions of Python don't handle automatically.
            (That is, you can't just slice ``Message.text`` with the offset and length.)

        Args:
            entity (:class:`telegram.MessageEntity`): The entity to extract the text from. It must
                be an entity that belongs to :attr:`description_entities`.

        Returns:
            :obj:`str`: The text of the given entity.

        Raises:
            RuntimeError: If the poll has no description.

        """
        if not self.description:
            raise RuntimeError("This Poll has no 'description'.")

        return parse_message_entity(self.description, entity)

    def parse_description_entities(
        self, types: list[str] | None = None
    ) -> dict[MessageEntity, str]:
        """
        Returns a :obj:`dict` that maps :class:`telegram.MessageEntity` to :obj:`str`.
        It contains entities from this polls description filtered by their ``type`` attribute as
        the key, and the text that each entity belongs to as the value of the :obj:`dict`.

        .. versionadded:: 22.8

        Note:
            This method should always be used instead of the :attr:`description_entities`
            attribute, since it calculates the correct substring from the message text based on
            UTF-16 codepoints. See :attr:`parse_description_entity` for more info.

        Args:
            types (list[:obj:`str`], optional): List of ``MessageEntity`` types as strings. If the
                    ``type`` attribute of an entity is contained in this list, it will be returned.
                    Defaults to :attr:`telegram.MessageEntity.ALL_TYPES`.
        Returns:
            dict[:class:`telegram.MessageEntity`, :obj:`str`]: A dictionary of entities
            mapped to the text that belongs to them, calculated based on UTF-16 codepoints.
        Raises:
            RuntimeError: If the poll has no description.
        """
        if not self.description:
            raise RuntimeError("This Poll has no 'description'.")

        return parse_message_entities(self.description, self.description_entities, types)

    @property
    def correct_option_id(self) -> int | None:
        """A zero based identifier of the correct answer
        option. Available only for closed polls in the quiz mode, which were sent
        (not forwarded), by the bot or to a private chat with the bot.

        .. deprecated:: 22.8
            Use :attr:`correct_option_ids` instead.
        """
        warn(
            PTBDeprecationWarning(
                "22.8",
                "The attribute `correct_option_id` is deprecated. "
                "Use `correct_option_ids` instead.",
            ),
            stacklevel=2,
        )
        return self.correct_option_ids[0] if self.correct_option_ids else None

    REGULAR: Final[str] = constants.PollType.REGULAR
    """:const:`telegram.constants.PollType.REGULAR`"""
    QUIZ: Final[str] = constants.PollType.QUIZ
    """:const:`telegram.constants.PollType.QUIZ`"""
    MAX_EXPLANATION_LENGTH: Final[int] = constants.PollLimit.MAX_EXPLANATION_LENGTH
    """:const:`telegram.constants.PollLimit.MAX_EXPLANATION_LENGTH`

    .. versionadded:: 20.0
    """
    MAX_EXPLANATION_LINE_FEEDS: Final[int] = constants.PollLimit.MAX_EXPLANATION_LINE_FEEDS
    """:const:`telegram.constants.PollLimit.MAX_EXPLANATION_LINE_FEEDS`

    .. versionadded:: 20.0
    """
    MIN_OPEN_PERIOD: Final[int] = constants.PollLimit.MIN_OPEN_PERIOD
    """:const:`telegram.constants.PollLimit.MIN_OPEN_PERIOD`

    .. versionadded:: 20.0
    """
    MAX_OPEN_PERIOD: Final[int] = constants.PollLimit.MAX_OPEN_PERIOD
    """:const:`telegram.constants.PollLimit.MAX_OPEN_PERIOD`

    .. versionadded:: 20.0
    """
    MIN_QUESTION_LENGTH: Final[int] = constants.PollLimit.MIN_QUESTION_LENGTH
    """:const:`telegram.constants.PollLimit.MIN_QUESTION_LENGTH`

    .. versionadded:: 20.0
    """
    MAX_QUESTION_LENGTH: Final[int] = constants.PollLimit.MAX_QUESTION_LENGTH
    """:const:`telegram.constants.PollLimit.MAX_QUESTION_LENGTH`

    .. versionadded:: 20.0
    """
    MIN_OPTION_LENGTH: Final[int] = constants.PollLimit.MIN_OPTION_LENGTH
    """:const:`telegram.constants.PollLimit.MIN_OPTION_LENGTH`

    .. versionadded:: 20.0
    """
    MAX_OPTION_LENGTH: Final[int] = constants.PollLimit.MAX_OPTION_LENGTH
    """:const:`telegram.constants.PollLimit.MAX_OPTION_LENGTH`

    .. versionadded:: 20.0
    """
    MIN_OPTION_NUMBER: Final[int] = constants.PollLimit.MIN_OPTION_NUMBER
    """:const:`telegram.constants.PollLimit.MIN_OPTION_NUMBER`

    .. versionadded:: 20.0
    """
    MAX_OPTION_NUMBER: Final[int] = constants.PollLimit.MAX_OPTION_NUMBER
    """:const:`telegram.constants.PollLimit.MAX_OPTION_NUMBER`

    .. versionadded:: 20.0
    """
    MAX_DESCRIPTION_CHARACTERS: Final[int] = constants.PollLimit.MAX_DESCRIPTION_CHARACTERS
    """:const:`telegram.constants.PollLimit.MAX_DESCRIPTION_CHARACTERS`

    .. versionadded:: 22.8
    """
    MIN_MEMBERSHIP_HOURS: Final[int] = constants.PollLimit.MIN_MEMBERSHIP_HOURS
    """:const:`telegram.constants.PollLimit.MIN_MEMBERSHIP_HOURS`

    .. versionadded:: 22.8
    """



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram Bot Access Settings."""

from collections.abc import Sequence
from typing import TYPE_CHECKING

from telegram._telegramobject import TelegramObject
from telegram._user import User
from telegram._utils.argumentparsing import de_list_optional, parse_sequence_arg
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot


class BotAccessSettings(TelegramObject):
    """
    This object describes the access settings of a bot.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`is_access_restricted` and :attr:`added_users` are equal.

    .. versionadded:: 22.8

    Args:
        is_access_restricted (:obj:`bool`): :obj:`True`, if only selected users can access the bot.
            The bot's owner can always access it.
        added_users (Sequence[:class:`telegram.User`], optional): The list of other users who
            have access to the bot if the access is restricted.

    Attributes:
        is_access_restricted (:obj:`bool`): :obj:`True`, if only selected users can access the bot.
            The bot's owner can always access it.
        added_users (Sequence[:class:`telegram.User`]): Optional. The list of other users who
            have access to the bot if the access is restricted.
    """

    __slots__ = ("added_users", "is_access_restricted")

    def __init__(
        self,
        is_access_restricted: bool,
        added_users: Sequence[User] | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.is_access_restricted: bool = is_access_restricted
        self.added_users: tuple[User, ...] = parse_sequence_arg(added_users)

        self._id_attrs = (self.is_access_restricted, self.added_users)
        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "BotAccessSettings":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)
        data["added_users"] = de_list_optional(data.get("added_users"), User, bot)

        return super().de_json(data=data, bot=bot)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains objects that represent managed bots in the Telegram Bot API."""

from typing import TYPE_CHECKING

from telegram._telegramobject import TelegramObject
from telegram._user import User
from telegram._utils.types import (
    JSONDict,
)

if TYPE_CHECKING:
    from telegram import Bot


class ManagedBotCreated(TelegramObject):
    """
    This object contains information about the bot that was created to be managed by the current
    bot.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`bot` is equal.

    .. versionadded:: 22.8

    Args:
        bot (:class:`telegram.User`): Information about the bot. The bot's token can be fetched
            using the method :meth:`~telegram.Bot.get_managed_bot_token`.
    Attributes:
        bot (:class:`telegram.User`): Information about the bot. The bot's token can be fetched
            using the method :meth:`~telegram.Bot.get_managed_bot_token`.
    """

    __slots__ = ("bot",)

    def __init__(
        self,
        bot: User,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.bot: User = bot
        self._id_attrs = (self.bot,)
        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "ManagedBotCreated":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data=data)

        data["bot"] = User.de_json(data=data["bot"], bot=bot)

        return super().de_json(data=data, bot=bot)


class ManagedBotUpdated(TelegramObject):
    """
    This object contains information about the creation, token update, or owner update of a bot
    that is managed by the current bot.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`user` and :attr:`bot` are equal.

    .. versionadded:: 22.8

    Args:
        user (:class:`telegram.User`): User that created the bot.
        bot (:class:`telegram.User`): Information about the bot. Token of the bot can be fetched
            using the method :meth:`~telegram.Bot.get_managed_bot_token`.

    Attributes:
        user (:class:`telegram.User`): User that created the bot.
        bot (:class:`telegram.User`): Information about the bot. Token of the bot can be fetched
            using the method :meth:`~telegram.Bot.get_managed_bot_token`.
    """

    __slots__ = ("bot", "user")

    def __init__(
        self,
        user: User,
        bot: User,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        # Required
        self.user: User = user
        self.bot: User = bot

        self._id_attrs = (
            self.user,
            self.bot,
        )
        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "ManagedBotUpdated":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data=data)

        data["user"] = User.de_json(data=data["user"], bot=bot)
        data["bot"] = User.de_json(data=data["bot"], bot=bot)

        return super().de_json(data=data, bot=bot)
